/*
/* desc.h copyright (c) 1996 Clemson University, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: desc.h,v $
 * Revision 1.10  1999/02/18 19:32:22  rbross
 * added sanity checking; it's defined as on right now...
 *
 * Revision 1.9  1998/11/10 22:44:25  rbross
 * *** empty log message ***
 *
 * Revision 1.8  1998/07/05 00:49:39  rbross
 * added FS_UNIX, FS_PVFS, FS_RESV defines, changed semantics of
 * fdesc's fs field
 *
 * Revision 1.7  1998/03/13 18:52:12  rbross
 * removed header string
 *
 * Revision 1.6  1997/10/28 19:29:20  pvfs
 * added license
 *
 * Revision 1.5  1997/07/10 18:39:26  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.4  1997/05/21 16:47:13  walt
 * fixed stupid typo
 *
 * Revision 1.3  1997/05/21 16:41:08  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1997/05/21 16:36:24  walt
 * modifed struct fdim to accomodate new multi-dimensional block code
 *  and buffered block code.
 *
 * Revision 1.1  1997/05/21 16:19:07  rmcettei
 * Initial revision
 *
/*/

#ifndef DESC_H
#define DESC_H

#include <meta.h>
#include <req.h>
#include <pvfs_config.h>

#define STRICT_DESC_CHECK

#ifdef STRICT_DESC_CHECK
#define IOD_INFO_CHECKVAL 123456789
#define FDESC_CHECKVAL    234567890
#endif

#define O_PASS  O_GADD /* passive connection */
#define O_SCHED O_GSTART /* scheduling connection */

/* FS TYPE DEFINES -- USED IN FDESC STRUCTUES */
#define FS_UNIX 0
#define FS_PVFS 1
#define FS_RESV 2

typedef struct fpart fpart, *fpart_p;

struct fpart {
	int offset;
	int gsize;
	int stride;
	int gstride;
	int ngroups;
};


typedef struct iod_info iod_info, *iod_info_p;

struct iod_info {
#ifdef STRICT_DESC_CHECK
	int checkval;
#endif
	struct sockaddr_in addr;
	iack ack;
	int sock;
};

typedef struct fdim fdim, *fdim_p;

struct fdim {
	int dims;	/* # of dimensions */
	int rcc, bc;
	int esize;	/* size of element */
	int dirty;	/* flags if buffer is dirty */
	char *buffer;	/* points to block buffer */
	int *buftag;	/* points to current buffer tag */
	int *bsize;	/* points to block size */
	int *bcount;	/* points to block count */
	int *bbuff;	/* points to buffering factor */
};

typedef struct fdesc fdesc, *fdesc_p;

struct fdesc {
#ifdef STRICT_DESC_CHECK
	int checkval;
#endif
	char fs; /* no longer boolean -- SEE DEFINES ABOVE! */
	fpart_p part_p;
	fdim_p dim_p;
	struct {
		off_t off;
		fmeta meta;
		int cap; /* capability */
		int ref; /* number of pfds referencing this structure */
		int flag; /* flags used to open file */
		int grp_nr; /* # in group (if using scheduled I/O) */
		iod_info iod[1];
	} fd;
};

#endif
