/* req.h copyright (c) 1996 Clemson University, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: req.h,v $
 * Revision 2.2  1999/06/17 15:41:27  rbross
 * added NOOP request for IOD
 *
 * Revision 2.1  1998/11/10 21:39:00  rbross
 * uses pvfs_config.h
 *
 * Revision 2.0  1998/11/03 19:41:15  rbross
 * added fs_ino field to iod requests to match new iod (v2.0)
 *
 * Revision 1.13  1998/08/18 18:10:24  spmcmil
 * Added utime request in mreq union.
 *
 * Revision 1.12  1998/07/23 21:58:37  rbross
 * added ftruncate, truncate, fsync, fdatasync requests etc.
 *
 * Revision 1.11  1998/07/21 04:18:42  rbross
 * updated fchown and fchmod mgr requests
 *
 * Revision 1.10  1998/07/14 18:39:24  rbross
 * added magic number support
 *
 * Revision 1.9  1998/03/13 18:52:12  rbross
 * removed header string
 *
 * Revision 1.8  1998/02/25 21:25:52  walt
 * added access system call
 *
 * Revision 1.7  1998/02/16 04:13:10  rbross
 * changed errno to eno as per Newt's glibc patches
 *
 * Revision 1.6  1997/10/28 19:32:10  pvfs
 * added license
 *
 * Revision 1.5  1997/07/10 18:38:27  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.4  1997/06/18 17:04:28  mcettei
 * add flags for iod_instr request
 *
 * Revision 1.3  1997/06/10 18:17:11  walt
 * updated to handle directories
 *
 * Revision 1.2  1997/06/05 14:49:21  rbross
 * i don't know what the hell is going on
 *
 * Revision 1.1  1997/01/29 20:59:47  rbross
 * Initial revision
 *
/*/

#ifndef REQ_H
#define REQ_H

#include <meta.h>
#include <utime.h>

/* defines for requests to manager */
#define MGR_MAJIK_NR 0x4a87c9fe

#define MGR_CHMOD    0
#define MGR_CHOWN    1
#define MGR_CLOSE    2
#define MGR_STAT     3
#define MGR_MOUNT    4
#define MGR_OPEN     5
#define MGR_UNLINK   6
#define MGR_SHUTDOWN 7
#define MGR_UMOUNT   8
#define MGR_FSTAT    9
#define MGR_RENAME   10
#define MGR_IOD_INFO 11
#define MGR_MKDIR    12
#define MGR_FCHOWN   13
#define MGR_FCHMOD   14
#define MGR_RMDIR    15
#define MGR_ACCESS   16
#define MGR_TRUNCATE 17
#define MGR_UTIME	   18

#define MAX_MGR_REQ  18

/* structure for request to manager */
typedef struct mreq mreq, *mreq_p;

struct mreq {
	int majik_nr;
	char type; /* type of request */
	int uid;   /* uid of program making request */
	int gid;   /* gid of program making request */
	int dsize; /* size of data following request */
	union {
		struct {
			char *fname; /* filename */
			fmeta meta; /* metadata for file (when creating?) */
			int   flag;
			int   mode; /* permissions when creating new file */
		} open;
		struct {
			char *fname;
			int owner;
			int group;
		} chown;
		struct {
			char *fname;
			mode_t mode;
		} chmod;	
		struct {
			fmeta meta;
		} close;
		struct {
			char *fname;
		} stat;	
		struct {
			char *fsname;
		} mount;
		struct {
			char *fsname;
		} umount;
		struct {
			char *fname;
		} unlink;	
		struct {
			fmeta meta;
		} fstat;	
		struct {
			char *oname;
			char *nname;
		} rename;
		struct {
			char* fsname;
		} iod_info;	
		struct {
			char* fname;
			mode_t mode;
		} mkdir;	
		struct {
			ino_t fs_ino; /* using fs_ino and file_ino instead of meta struct */
			ino_t file_ino;
			int owner;
			int group;
		} fchown;
		struct {
			ino_t fs_ino; /* using fs_ino and file_ino instead of meta struct */
			ino_t file_ino;
			mode_t mode;
		} fchmod;	
		struct {
			char* fname;
		} rmdir;	
		struct {
			char* fname;
			mode_t mode;
		} access;	
		struct {
			char *fname;
			size_t length;
		} truncate;
		struct {
			char *fname;
			struct utimbuf buf;
		} utime;
	} req;
};

/* structure for ack from manager */
typedef struct mack mack, *mack_p;

struct mack {
	int majik_nr;
	char type;
	int status;
	int eno;
	int dsize;
	union {
		struct {
			fmeta meta; /* metadata of opened file */
			int cap;
		} open;
		struct {
			fmeta meta; /* metadata of opened file */
		} stat;
		struct {
			fmeta meta; /* metadata requested */
		} fstat;	
		struct {
			int nr_iods;
		} iod_info;
	} ack;
};

/* defines for requests to iod */
#define IOD_MAJIK_NR 0x49e3ac9f
#define IOD_OPEN     0
#define IOD_CLOSE    1
#define IOD_STAT     2
#define IOD_UNLINK   3
#define IOD_RW       4
#define IOD_SHUTDOWN 5

/* defines for group requests */
#define IOD_GSTART   6
#define IOD_GADD     7

/* defines for instrumentation */
#define IOD_INSTR		8

/* more defines for iod requests */
#define IOD_FTRUNCATE 9
#define IOD_TRUNCATE  10
#define IOD_FDATASYNC 11 /* also used for fsync() */

/* a no-op request, for opening a connection w/out a request */
#define IOD_NOOP     12

/* subtypes for RW requests */
#define IOD_RW_READ  0
#define IOD_RW_WRITE 1

#define MAX_IOD_REQ  12

/* flags for instr request */
#define INSTR_ON		1
#define INSTR_OFF		2
#define INSTR_PRNT 	4
#define INSTR_RST		8

/* structure for req to iod */
typedef struct ireq ireq, *ireq_p;

struct ireq {
	int majik_nr;
	char type;
	int dsize;
	union {
		struct {
			ino_t fs_ino;
			ino_t f_ino; /* inode # of metadata file */
			int   cap;
			int   flag; /* flags for opening */
			int   mode; /* permissions when creating new file */
			pvfs_stat p_stat; /* info on striping of file */
			int   pnum; /* partition # for this file */
		} open;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
			int cap;
		} close;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
		} unlink;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
		} stat;
		struct {
			char rw_type;  /* subtype - what kind of access */
			ino_t fs_ino;
			ino_t f_ino;   /* file (metadata file) inode # */
			int cap;
			int off;       /* initial offset */
			int fsize;     /* first size */
			int gsize;     /* group size */
			int stride;    /* stride */
			int gcount;    /* group count */
			int lsize;     /* last size */
			int gstride;   /* group stride */
			int ngroups;   /* # of groups */
			int c_nr;      /* connection # (for indirect I/O) */
		} rw;
		struct {
			ino_t fs_ino;
			ino_t f_ino;   /* inode # of metadata file */
			int cap;       /* capability of scheduler */
			int grp_nr;    /* suggested group # */
		} gadd;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
			int cap;
			int grp_sz;    /* size (# of members) of group */
		} gstart;
		struct {
			int flags;		/* flags for information to return */
		} instr;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
			int cap;
			int length;
		} ftruncate;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
			pvfs_stat p_stat;
			int length;
			int part_nr;
		} truncate;
		struct {
			ino_t fs_ino;
			ino_t f_ino;
			int cap;
		} fdatasync;
	} req;
};

typedef struct iack iack, *iack_p;

struct iack {
	int majik_nr;
	char type;
	int status;
	int eno;
	int dsize;
	union {
		struct {
			int fsize;
		} stat;
		struct {
			int grp_nr;   /* actual group # */
			int grp_sz;   /* size of group */
		} gadd;
		struct {
			float avg;
			float wait;
			int	accesses;
			float dev;
		} instr;
	} ack;
};

#endif
