/*
 * alist.h copyright (c) 1997 Clemson University, all rights reserved.
 *
 * Written by Rob Ross and Matt Cettei.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.eng.clemson.edu
 *           Matt Cettei mcettei@parl.eng.clemson.edu
 */


/*
 * $Log: alist.h,v $
 * Revision 1.9  1997/11/13 22:26:15  rbross
 * added sockopt access type
 *
 * Revision 1.8  1997/03/22 22:20:27  rbross
 * added prototype for a_type_search
 *
 * Revision 1.7  1997/03/14 18:14:16  rbross
 * added prototypes for a_ptr_cmp and a_get_start
 *
 * Revision 1.6  1997/03/14 05:53:52  rbross
 * moved access type defines here
 *
 * Revision 1.5  1997/03/13 21:46:50  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1996/11/26 22:22:42  rbross
 * added alist_empty prototype
 *
 * Revision 1.3  1996/11/07 20:01:00  rbross
 * *** empty log message ***
 *
 * Revision 1.2  1996/11/07 19:34:16  rbross
 * changed type some
 *
 * Revision 1.1  1996/10/31 21:04:20  rbross
 * Initial revision
 *
/*/

#ifndef ALIST_H
#define ALIST_H

#include <llist.h>
#include <flist.h>

static char alist_h[] = "$Header: /projects/pvfs/pvfs-working/iod/RCS/alist.h,v 1.9 1997/11/13 22:26:15 rbross Exp $";

/* access defines */
#define A_NOP 0
#define A_ACK 1
#define A_READ 2
#define A_WRITE 3
#define A_LIST 4
#define A_SOCKOPT 5

#define a_search(__al_p, __sock) a_sock_search((__al_p), (__sock))

typedef llist_p alist_p;

typedef struct ainfo ainfo, *ainfo_p;

struct ainfo {
	int type;
	int sock;
	union {
		struct {
			int off;
			int size;
			finfo_p file_p; /* pointer to open file info */
		} rw;
		struct {
			void *ack_p;
			void *cur_p;
			int size;
		} ack;
		struct {
			alist_p al_p;
		} list;
		struct {
			int level;
			int name;
			int val;
		} sockopt;
	} u;
};

alist_p alist_new(void);
int alist_empty(alist_p);
int a_add_start(alist_p, ainfo_p);
int a_add_end(alist_p, ainfo_p);
ainfo_p a_sock_search(alist_p, int sock);
ainfo_p a_type_search(alist_p, int type);
ainfo_p a_get_start(alist_p al_p);
int a_sock_rem(alist_p, int sock);
int a_ptr_rem(alist_p, ainfo_p);
int a_ptr_cmp(void *key, void *a_p);
void alist_cleanup(alist_p);
int a_dump(void *);
int alist_dump(alist_p al_p);
int ainfo_new(ainfo_p);

#endif
