/*
/* iod_group.c copyright (c) 1997 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/


/*
 * $Log: iod_group.c,v $
 * Revision 1.3  1998/02/16 04:20:18  rbross
 * added Newt's glibc patches
 *
 * Revision 1.2  1997/10/29 21:42:55  pvfs
 * added license
 *
 * Revision 1.1  1997/03/22 22:24:19  rbross
 * Initial revision
 *
/*/

static char iod_group_c[] = "$Header: /projects/pvfs/pvfs-working/iod/RCS/iod_group.c,v 1.3 1998/02/16 04:20:18 rbross Exp $";

/* INCLUDES */
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <flist.h>
#include <jlist.h>
#include <alist.h>
#include <req.h>
#include <sockset.h>
#include <iod.h>
#include <debug.h>

/* GLOBALS */
extern jlist_p jobs_p; /* list of active jobs */
extern flist_p open_p; /* list of open files */
extern sockset socks;

/* PROTOTYPES */
static int send_group_acks(finfo_p f_p);

/* FUNCTIONS */
int do_gstart_req(int sock, ireq_p r_p, iack_p a_p) {
	finfo_p f_p;
	int i, gsz = r_p->req.gstart.grp_sz;

	LOG2("do_gstart_req: request for ino = %d, cap = %d\n",
		r_p->req.gstart.f_ino, r_p->req.gstart.cap);

	/* find file (and cap) in list */
	if (!(f_p = f_search(open_p, r_p->req.gstart.f_ino,
	r_p->req.gstart.cap))) {
		return(send_error(-1, ENOENT, sock, a_p));
	}

	if (f_p->grp.sched >= 0) /* already have a scheduler? */ {
		ERR("do_gstart_req: scheduler socket already set\n");
		return(send_error(-1, EINVAL, sock, a_p));
	}

	f_p->grp.sched = sock;

	if (!f_p->grp.conn) /* need to allocate space */ {
		if (!(f_p->grp.conn = (int *)malloc(sizeof(int)*gsz))) return(-1);
		for (i=0; i < gsz; i++) f_p->grp.conn[i] = -1;
		f_p->grp.nr_conn = gsz;
		return(0); /* !space => !done */
	}

	if (f_p->grp.conn && f_p->grp.nr_conn > gsz) /* too much already? */ {
		ERR("do_gstart_req: a group member has too high a number\n");
		return(send_error(-1, EINVAL, sock, a_p));
	}

	if (f_p->grp.conn && f_p->grp.nr_conn < gsz) /* need to reallocate */ {
		int old_gsz = f_p->grp.nr_conn;
		if (!(f_p->grp.conn = (int *)realloc(f_p->grp.conn,
			sizeof(int)*gsz))) return(-1);
		for (i=old_gsz; i < gsz; i++) f_p->grp.conn[i] = -1;
		f_p->grp.nr_conn = gsz;
		return(0); /* we expanded group size, so !done */
	}

	for (i=0; i < f_p->grp.nr_conn; i++) if (f_p->grp.conn[i] < 0) break;

	if (i == f_p->grp.nr_conn) /* done */ send_group_acks(f_p);
	return(0);
}

int do_gadd_req(int sock, ireq_p r_p, iack_p a_p) {
	finfo_p f_p;
	int i, nr = r_p->req.gadd.grp_nr;

	LOG3("do_gadd_req: request for ino = %d, cap = %d on sock %d\n",
		r_p->req.gadd.f_ino, r_p->req.gadd.cap, sock);

	/* find file (and cap) in list */
	if (!(f_p = f_search(open_p, r_p->req.gstart.f_ino,
		r_p->req.gstart.cap)))
	{
		ERR("do_gadd_req: f_search failed\n");
		return(send_error(-1, ENOENT, sock, a_p));
	}

	/* remember: !sched => !done, sched => space */
	if (!f_p->grp.conn) /* need to allocate space */ {
		if (nr < 0 || nr > 99) {
			/* ridiculous grp_nr; send an error */
			ERR1("do_gadd_req: grp_nr is stupid value (%d)\n", nr);
			return(send_error(-1, EINVAL, sock, a_p));
		}

		/* assume group size is nr+1 */
		if (!(f_p->grp.conn = (int *)malloc(sizeof(int)*nr+1))) return(-1);
		for (i=0; i < nr; i++) f_p->grp.conn[i] = -1;
		f_p->grp.conn[nr] = sock;
		f_p->grp.nr_conn = nr+1;

		/* !space => !sched, so we know we're not ready to return acks */
		return(0);
	}

	/* we already have space */
	if (nr >= f_p->grp.nr_conn) /* need more space */ {
		int old_gsz = f_p->grp.nr_conn;
		if (f_p->grp.sched >= 0) /* group size is set correctly */ {
			ERR1("do_gadd_req: grp_nr too large (%d)\n", nr);
			return(send_error(-1, EINVAL, sock, a_p));
		}

		/* assume group size should be nr+1; reallocate */
		if (!(f_p->grp.conn = (int *)realloc(f_p->grp.conn,
			sizeof(int)*nr+1))) return(-1);
		for (i=old_gsz; i < nr; i++) f_p->grp.conn[i] = -1;
		f_p->grp.conn[nr] = sock;
		f_p->grp.nr_conn = nr+1;
		/* !sched => !done, so we're not ready to return acks */
		return(0);
	}

	/* wow, we have space AND the group number passed in is ok */
	if (f_p->grp.conn[nr] >= 0) /* already used */ {
		ERR1("do_gadd_req: grp_nr already used (%d)\n", nr);
		return(send_error(-1, EINVAL, sock, a_p));
	}
	f_p->grp.conn[nr] = sock;

	if (f_p->grp.sched < 0) /* !sched */ return(0);
	for (i=0; i < f_p->grp.nr_conn; i++) if (f_p->grp.conn[i] < 0) break;

	if (i == f_p->grp.nr_conn) /* done */ send_group_acks(f_p);
	return(0);
}

static int send_group_acks(finfo_p f_p)
{
	int i;
	jinfo_p j_p;
	ainfo_p acc_p;
	iack_p  ack_p;

	if (!(j_p = j_new())) return(-1);
	for (i=0; i < f_p->grp.nr_conn; i++) /* add acks to group members */ {
		/* set up ack access */
		if (!(acc_p = ack_a_new())) return(-1);
		ack_p = (iack_p)acc_p->u.ack.ack_p;

		ack_p->type = IOD_GADD;
		ack_p->status = 0;
		ack_p->eno  = 0;
		ack_p->dsize  = 0;
		ack_p->ack.gadd.grp_nr = i;
		ack_p->ack.gadd.grp_sz = f_p->grp.nr_conn;

		acc_p->sock = f_p->grp.conn[i];

		/* add ack access to job and set ready to write */
		if (a_add_end(j_p->al_p, acc_p) < 0) return(-1);
		FD_SET(f_p->grp.conn[i], &j_p->socks);
		addsockwr(f_p->grp.conn[i], &socks);
	}

	if (!(acc_p = ack_a_new())) return(-1);
	ack_p = (iack_p)acc_p->u.ack.ack_p;

	ack_p->type = IOD_GSTART;
	ack_p->status = 0;
	ack_p->eno  = 0;
	ack_p->dsize  = 0;

	acc_p->sock = f_p->grp.sched;

	/* add ack access to job and set ready to write */
	if (a_add_end(j_p->al_p, acc_p) < 0) return(-1);
	FD_SET(f_p->grp.sched, &j_p->socks);
	addsockwr(f_p->grp.sched, &socks);

	if (j_add(jobs_p, j_p) < 0) {
		ERR("send_group_acks: j_add failed\n");
		j_free(j_p);
		return(-1);
	}
	return(0);
}
