/*
/* alist.h copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
/*/

/*
/* ALIST.H - access list header file FOR LIBRARY CODE
/*
/*/

/*
 * $Log: alist.h,v $
 * Revision 1.7  1998/03/13 18:53:29  rbross
 * removed header string
 *
 * Revision 1.6  1997/10/27 20:58:50  pvfs
 * added license
 *
 * Revision 1.5  1997/03/18 00:00:35  rbross
 * added prototype for a_get_start()
 *
 * Revision 1.4  1997/03/13 21:44:38  mcettei
 * *** empty log message ***
 *
 * Revision 1.3  1997/03/13 19:19:56  mcettei
 * added new_ainfo prototype
 *
 * Revision 1.2  1997/02/26 20:45:22  rbross
 * *** empty log message ***
 *
 * Revision 1.1  1997/01/14 21:54:30  rbross
 * Initial revision
 *
/*/

#ifndef ALIST_H
#define ALIST_H

#include <llist.h>

typedef llist_p alist_p;

typedef struct ainfo ainfo, *ainfo_p;

#define A_READ  0
#define A_WRITE 1
#define A_REQ   2
#define A_ACK   3

struct ainfo {
	int type;
	int sock;
	union {
		struct {
			char *loc;
			int size;
		} rw;
		struct {
			char *req_p;
			char *cur_p;
			int size;
		} req;
		struct {
			int iod_nr;
			char *ack_p;
			char *cur_p;
			int size;
		} ack;
		struct {
			alist_p al_p;
		} list;
	} u;
};

alist_p alist_new(void);
int a_add_start(alist_p, ainfo_p);
int a_add_end(alist_p, ainfo_p);
ainfo_p a_get_start(alist_p);
ainfo_p a_search(alist_p, int sock);
int a_sock_rem(alist_p, int sock);
int a_ptr_rem(alist_p, ainfo_p);
void a_cleanup(alist_p);
int a_dump(void *);
int alist_dump(alist_p al_p);
int ainfo_new(ainfo_p);

#endif
