/*
/* jlist.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  rbross@parl.eng.clemson.edu  or
				 Matt Cettei mcettei@parl.eng.clemson.edu
/*/

/*
/* JLIST.C - functions to handle the creation and modification of 
/*
/* This is built on top of the llist.[ch] files
/* Note: this file is used both in the iod and in library functions
/*
/*/

/*
 * $Log: jlist.c,v $
 * Revision 1.9  1999/06/17 15:36:46  rbross
 * *** empty log message ***
 *
 * Revision 1.8  1997/10/27 21:01:28  pvfs
 * added license
 *
 * Revision 1.7  1997/03/18 00:05:28  rbross
 * moved new_jinfo() to j_new(); modified to use fd_set for socket list
 * for job instead of a single socket
 *
 * Revision 1.6  1997/03/14 05:55:36  rbross
 * fixed some bad return values and problems w/new_jinfo()
 *
 * Revision 1.5  1997/03/13 22:31:15  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1997/03/13 21:45:16  mcettei
 * *** empty log message ***
 *
 * Revision 1.3  1997/03/13 19:11:09  mcettei
 * add new_jinfo
 *
 * Revision 1.2  1997/02/19 18:43:32  rbross
 * outputs data on stderr now
 *
 * Revision 1.1  1997/02/07 21:11:51  rbross
 * Initial revision
 *
 * Revision 1.5  1996/12/02 19:06:56  mcettei
 * removed alist_cleanup from jfree
 *
 * Revision 1.4  1996/11/18 21:59:30  rbross
 * added jlist_empty
 *
 * Revision 1.3  1996/11/07 19:53:49  rbross
 * *** empty log message ***
 *
 * Revision 1.2  1996/11/07 17:09:26  rbross
 * *** empty log message ***
 *
 * Revision 1.1  1996/10/31 21:04:31  rbross
 * Initial revision
 *
/*/

#include <stdlib.h>
#include <jlist.h>
#include <debug.h>

static char jlist_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/jlist.c,v 1.9 1999/06/17 15:36:46 rbross Exp $";

/* prototypes for internal functions */
static int j_sock_cmp(void *key, void *j_p);
static void j_free(void *j_p);

jlist_p jlist_new(void)
{
	return(llist_new());
}

int jlist_empty(jlist_p jl_p)
{
	return(llist_empty(jl_p));
}

int j_add(jlist_p jl_p, jinfo_p j_p)
{
	return(llist_add(jl_p, (void *) j_p));
}

jinfo_p j_search(jlist_p jl_p, int sock)
{
	return((jinfo_p) llist_search(jl_p, (void *) sock, j_sock_cmp));
}

int j_rem(jlist_p jl_p, int sock)
{
	jinfo_p j_p;
	
	j_p = (jinfo_p) llist_rem(jl_p, (void *)sock, j_sock_cmp);
	if (j_p) {
		j_free(j_p);
		return(0);
	}
	return(-1);
}

static int j_sock_cmp(void *key, void *j_p)
{
#if 0
	if ((int) key == ((jinfo_p) j_p)->sock) return(0);
#else
	if (FD_ISSET((int) key, &((jinfo_p) j_p)->socks)) return(0);
#endif
	else return(1);
}

void jlist_cleanup(jlist_p jl_p)
{
	llist_free(jl_p, j_free);
}

/* j_free() - frees the memory allocated to store a job */
static void j_free(void *j_p)
{
	alist_cleanup(((jinfo_p) j_p)->al_p);
	free((jinfo_p) j_p);
}

int jlist_dump(jlist_p jl_p)
{
	return(llist_doall(jl_p, j_dump));
}

int j_dump(void *v_p)
{
	jinfo_p j_p = (jinfo_p) v_p;
	
	LOG2("type: %d\nsize: %d\n", j_p->type, j_p->size);
	alist_dump(j_p->al_p);
}

jinfo_p j_new(int pcount) 
{
	jinfo_p j_p;
	int i;

	if ((j_p = (jinfo_p) malloc(sizeof(jinfo)
		+ sizeof(int)*(pcount-1))) == NULL) {
		return(0);
	}
	if (!(j_p->al_p = alist_new())) /* error creating access list */ {
		free(j_p);
		return(0);
	}

	j_p->type = 0;
#if 0
	j_p->sock = -1;
#else
	j_p->max_nr = 0;
	FD_ZERO(&j_p->socks);
#endif
	j_p->size = 0; /* new job - no accesses */
	for (i=0; i < pcount; i++) j_p->psize[i] = 0;
	return(j_p);
}




