/*
/* pvfs_access.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* PVFS_ACCESS.C - 
/*
/*/

/*
 * $Log: pvfs_access.c,v $
 * Revision 1.5  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.4  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.2  1998/03/13 22:28:17  rbross
 * added unix_access() call - missing?!?
 *
 * Revision 1.1  1998/02/25 21:26:22  walt
 * Initial revision
 *
 *
/*/

static char pvfs_access_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_access.c,v 1.5 1998/07/10 20:20:00 rbross Exp $";

/* This file contains PVFS library call for access.   		*/
/* It will determine if file is UNIX or PVFS and then 		*/
/* make the proper request.					*/

#include <lib.h>
#include <meta.h>

extern int errno;
int unix_access(char *path, int mode);

int pvfs_access(char* pathname, int mode)
{
	int i, s;
	char host[256];
	mack ack;
	mreq request;
	char *t_p;
	dmeta dir;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	/* check for PVFS files/dirs */
	t_p = pathname + strlen(pathname);
	while (*t_p != '/' && t_p > pathname) t_p--;
	
	if ((i = check_pvfs(pathname)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return(syscall(SYS_access, pathname, mode));
	}
	if ((i = get_dmeta(pathname, &dir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	*(dir.sd_path-1) = '/';
	*(dir.fname-1) = '/';
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_ACCESS;
	request.dsize = strlen(dir.rd_path);
	request.req.access.fname = 0;
	request.req.access.mode = mode;

	/* Send request to mgr */
	if ((s = send_mreq_dmeta(&dir, &request, dir.rd_path, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return(0);
}
