/*
/* pvfs_chown.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  rbross@parl.eng.clemson.edu  or
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_chown.c,v $
 * Revision 1.12  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.11  1998/07/09 17:14:44  rbross
 * *** empty log message ***
 *
 * Revision 1.8  1997/10/27 21:08:13  pvfs
 * added license
 *
 * Revision 1.7  1997/07/10 18:34:51  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.6  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.5  1997/01/20 17:12:31  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1997/01/16 19:28:24  mcettei
 * uses debugging code
 *
 * Revision 1.3  1996/12/06 20:55:57  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1996/12/05 19:07:27  mcettei
 * added header
 *
 *
/*/
static char pvfs_chown_c[] = "$Header:
/projects/pvfs2/lib/RCS/pvfs_chown.c,v 1.3 1996/12/05 19:06:03 mcettei Exp $";

/* This file contains PVFS library call for chown.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_chown(const char* pathname, uid_t owner, gid_t group)
{
int i, flag, s;
int succ;
mreq request;
mack ack;
char host[256];
dmeta dir;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	/* Check to see if .pvfsdir exists in directory  	*/
	if ((i = check_pvfs(pathname)) < 1) {
		if (i < 0) {
			PERROR("pvfs_chown: finding file");
			return(-1);
		}	
		return (syscall(SYS_chown, pathname, owner, group));
	}
	if ((i = get_dmeta(pathname, &dir)) < 0) {
		PERROR("pvfs_chown: getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	*(dir.sd_path-1) = '/';
	*(dir.fname-1) = '/';
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_CHOWN;
	request.dsize = strlen(dir.rd_path);
	request.req.chown.fname = 0;
	request.req.chown.owner = (int)owner;
	request.req.chown.group = (int)group;

	/* Send request to mgr */	
	if ((s = send_mreq_dmeta(&dir, &request, dir.rd_path, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return (0);
}
