/* pvfs_fstat.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu
/*/

/*
 * $Log: pvfs_fstat.c,v $
 * Revision 1.17  1999/06/17 15:36:46  rbross
 * cleanup
 *
 * Revision 1.16  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.14  1998/02/21 03:29:16  rbross
 * yet another stat fix
 *
 * Revision 1.13  1998/02/19 23:03:06  rbross
 * removed SYS_stat defines ... moving to lib.h
 *
 * Revision 1.12  1998/02/16 22:03:56  rbross
 * repatched to work with libc5 too
 *
 * Revision 1.11  1998/02/16 19:03:10  rbross
 * Newt patches
 *
 * Revision 1.10  1997/10/27 21:14:47  pvfs
 * added license
 *
 * Revision 1.9  1997/06/26 21:53:36  walt
 * fixed problem fstat-ing non-pvfs file
 *
 * Revision 1.8  1997/02/26 15:48:27  rbross
 * update to use newer fstat system call
 *
 * Revision 1.7  1997/02/11 22:10:50  mcettei
 * removed pvfs_fd union
 *
 * Revision 1.6  1997/01/20 17:14:16  mcettei
 * *** empty log message ***
 *
 * Revision 1.5  1997/01/16 19:31:58  mcettei
 * uses debugging code
 *
 * Revision 1.4  1997/01/15 19:24:20  mcettei
 * works correctly now, uses mgr call
 *
 * Revision 1.3  1996/12/06 20:51:56  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1996/12/06 20:46:14  mcettei
 * added header
 *
 *
/*/

static char pvfs_fstat_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_fstat.c,v 1.17 1999/06/17 15:36:46 rbross Exp $";

/* This file contains PVFS library call for fstat.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <sys/syscall.h>

extern int errno;
extern fdesc_p pfds[];

int pvfs_fstat(int fd, struct stat *buf)
{
	int i, s;
	struct sockaddr saddr;
	fmeta meta;
	mack ack;
	mreq req;

	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  

	if (!pfds[fd] || pfds[fd]->fs == FS_UNIX) {
		return(unix_fstat(fd, buf));
	}	

	/* prepare request */
	req.uid  = getuid();
	req.gid  = getgid();
	req.type = MGR_FSTAT;
	req.dsize= 0;
	req.req.fstat.meta = pfds[fd]->fd.meta;

	/* send request to mgr */
	saddr = pfds[fd]->fd.meta.mgr;
	if ((s = send_mreq_saddr(&saddr, &req, NULL, &ack)) < 0) {
		PERROR("send_mreq_saddr");
		return(-1);
	}

	/* don't close manager connection */

	*buf = ack.ack.fstat.meta.u_stat;
	return (0);
}
