/*
/* pvfs_gcalls.c copyright (c) 1996 R. Ross and M. Cettei, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_gcalls.c,v $
 * Revision 1.6  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.4  1998/02/16 19:04:19  rbross
 * Newt patches
 *
 * Revision 1.3  1997/10/27 21:15:09  pvfs
 * added license
 *
 * Revision 1.2  1997/03/22 22:32:26  rbross
 * working version; added more retry capability
 * should rewrite to remove iod-related retry; no longer needed.
 *
 * Revision 1.1  1997/03/18 00:07:32  rbross
 * Initial revision
 *
/*/

static char header[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_gcalls.c,v 1.6 1998/07/07 16:48:33 rbross Exp $";

#include <lib.h>

/* EXTERNS */
extern jlist_p active_p;
extern fdesc_p pfds[];
extern sockset socks;

/* FUNCTIONS */
#define PCNT pfds[fd]->fd.meta.p_stat.pcount
#define FINO pfds[fd]->fd.meta.u_stat.st_ino

int pvfs_gstart(int fd, int grp_sz)
{
	ireq req;
	int i;

	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  

	if (!pfds[fd] || pfds[fd]->fs == FS_UNIX) /* no fd data or UNIX file */ {
		errno = EBADF;
		return(-1);
	}

	if (grp_sz < 0) /* bad group size */ {
		errno = EINVAL;
		return(-1);
	}

	for(i=0; i < PCNT; i++) {
		pfds[fd]->fd.iod[i].ack.status = 0;
		pfds[fd]->fd.iod[i].ack.dsize  = 0;
	}

	req.type              = IOD_GSTART;
	req.dsize             = 0;
	req.req.gstart.f_ino  = FINO;
	req.req.gstart.cap    = pfds[fd]->fd.cap;
	req.req.gstart.grp_sz = grp_sz;

	/* build job to send reqs and recv acks */
	if (build_simple_jobs(pfds[fd], &req) < 0) {
		ERR("pvfs_gstart: build_simple_jobs failed\n");
		return(-1);
	}

	/* call do_job */
	while (!jlist_empty(active_p)) {
		if (do_jobs(active_p, &socks, -1) < 0) {
			ERR("pvfs_gstart: do_jobs failed\n");
			return(-1);
		}
	}

	/* check acks from iods */
	for (i=0; i < PCNT; i++) {
		if (pfds[fd]->fd.iod[i].ack.status) {
			ERR1("pvfs_gstart: non-zero status returned from iod %d\n", i);
			errno = pfds[fd]->fd.iod[i].ack.eno;
			return(-1);
		}
	}
	return(0);
}

int pvfs_gadd(int fd, int grp_nr)
{
	ireq req;
	int i;
	char retry = 10;

	if (!pfds[fd] || pfds[fd]->fs == FS_UNIX) /* no fd data or UNIX file */ {
		errno = EBADF;
		return(-1);
	}

	for(i=0; i < PCNT; i++) {
		pfds[fd]->fd.iod[i].ack.status = 0;
		pfds[fd]->fd.iod[i].ack.dsize  = 0;
	}

	req.type            = IOD_GADD;
	req.dsize           = 0;
	req.req.gadd.f_ino  = FINO;
	req.req.gadd.cap    = pfds[fd]->fd.cap;
	req.req.gadd.grp_nr = grp_nr;

	for (;;) {
	FOO:
		/* build job to send reqs and recv acks */
		if (build_simple_jobs(pfds[fd], &req) < 0) {
			ERR("pvfs_gadd: build_simple_jobs failed\n");
			return(-1);
		}

		/* call do_job */
		while (!jlist_empty(active_p)) {
			if (do_jobs(active_p, &socks, -1) < 0) {
				ERR("pvfs_gadd: do_jobs failed\n");
				return(-1);
			}
		}

		/* check acks from iods */
		for (i=0; i < PCNT; i++) {
			if (pfds[fd]->fd.iod[i].ack.status) {
				if (pfds[fd]->fd.iod[i].ack.eno == ENOSPC && retry) {
					LOG("pvfs_gadd: group not yet started; retrying\n");
					retry--;
					sleep(2);
					goto FOO; /* haha!  get some Dykstra! */
				}
				errno = pfds[fd]->fd.iod[i].ack.eno;
				ERR1("pvfs_gadd: non-zero status returned from iod %d\n", i);
				return(-1);
			}
		}
		/* should check to see that we got the same # back from each...  */
		pfds[fd]->fd.grp_nr = pfds[fd]->fd.iod[0].ack.ack.gadd.grp_nr;
		LOG1("pvfs_gadd: setting group number to %d\n", pfds[fd]->fd.grp_nr);
		return(0);
	}
	return(-1);
}

int sched_read(int fd, size_t count)
{
	LOG2("sched_read: fd = %d, sz = %d\n", fd, count);
	return(sched_rw(fd, count, J_READ));
}

int sched_write(int fd, size_t count)
{
	LOG2("sched_write: fd = %d, sz = %d\n", fd, count);
	return(sched_rw(fd, count, J_WRITE));
}

int sched_rw(int fd, size_t count, int jtype)
{
	ireq req;
	fdesc_p pfd_p = pfds[fd];
	int i, size = 0;

	if (!pfd_p || pfd_p->fs == FS_UNIX) /* no fd data or UNIX file */ {
		errno = EBADF;
		return(-1);
	}

	if (!pfds[fd]->part_p) /* no partition set! */ {
		ERR("sched_rw: warning: no partition set\n");
#if 0
		errno = EBADF;
		return(-1);
#endif
	}

	/* build a request as if we are the first CN; this fills in:
	 * offset, [fgl]size, gcount, stride
	/*/
	if (build_rw_req(pfd_p, count, &req, jtype, 0) < 0) {
		ERR("build_rw_req failed\n");
		return(-1);
	}

	/* fill in the rest of the parameters */
	req.req.rw.ngroups = -1; /* not saved @ sched. and not used by iod */
	if (!pfds[fd]->part_p) /* no partition set */ {
		req.req.rw.gstride = 0;
	}
	else {
		req.req.rw.gstride = pfd_p->part_p->gstride;
	}

	/* build job to send reqs and recv acks */
	if (build_simple_jobs(pfd_p, &req) < 0) {
		ERR("sched_rw: build_simple_jobs failed\n");
		return(-1);
	}

	/* call do_job */
	while (!jlist_empty(active_p)) {
		if (do_jobs(active_p, &socks, -1) < 0) {
			ERR("sched_rw: do_jobs failed\n");
			return(-1);
		}
	}

	/* check acks from iods */
	for (i=0; i < PCNT; i++) {
		if (pfd_p->fd.iod[i].ack.status) {
			ERR1("sched_rw: non-zero status returned from iod %d\n", i);
			errno = pfd_p->fd.iod[i].ack.eno;
			return(-1);
		}
		size += pfd_p->fd.iod[i].ack.dsize;
	}
	
	/* update file pointer; increment by count if we accessed more than
	 * count bytes
	 */
	if (size < count) pfd_p->fd.off += size;
	else pfd_p->fd.off += count;
	return(size);
}






