/*
/* pvfs_instr.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* PVFS_INSTR.C - function to perform read access to files
/*
/*/

/*
 * $Log: pvfs_instr.c,v $
 * Revision 1.6  1999/06/17 15:36:46  rbross
 * *** empty log message ***
 *
 * Revision 1.5  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.4  1998/02/16 19:05:26  rbross
 * Newt patches
 *
 * Revision 1.3  1997/10/27 21:12:56  pvfs
 * added license
 *
 * Revision 1.2  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.1  1997/06/04 14:12:08  walt
 * Initial revision
 *
 *
/*/

static char pvfs_instr_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_instr.c,v 1.6 1999/06/17 15:36:46 rbross Exp $";

#include <lib.h>
#include <meta.h>
#include <fslist.h>

extern jlist_p active_p;
extern sockset socks;
extern int errno;

int pvfs_instr(char *dir, int flags)
{
int s, i, ret;
char host[256];
mreq req;
mack ack;
ireq iodreq;
iack iodack;
fsinfo_p fs_p;
dmeta dotdir;
	
	if ((i = get_dmeta(dir, &dotdir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}
/* Prepare request for file system  */
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_IOD_INFO;
	req.dsize = strlen(dir);
	req.req.iod_info.fsname = 0;
/* Still need to get socket address s */
	if ((s = new_sock()) < 0) {
		PERROR("new_sock"); return(-1);
	}
	if (connect_sock(s, dotdir.host, dotdir.port) < 0) {
		PERROR("connect_sock"); return(-1);
	}
	if (bsend(s, &req, sizeof(mreq)) < sizeof(mreq)) {
      PERROR("bsend (req)"); return(-1);
   }
   if (req.dsize) if (bsend(s, dir, req.dsize) < req.dsize) {
      PERROR("bsend (data)"); return(-1);
   }
   if ((i = brecv(s, &ack, sizeof(ack))) < 0) {
      PERROR("pvfs_instr: brecv"); return(-1);
   }
	/* Check ack */
	if (ack.status) {
		errno = ack.eno;
		close(s);
		return(-1);
	}	

	fs_p = (fsinfo_p)malloc(sizeof(fsinfo) + sizeof(iod_info) * 
		(ack.ack.iod_info.nr_iods - 1));
   if ((i = brecv(s, fs_p->iod, ack.dsize)) < 0) {
      PERROR("pvfs_instr: brecv");
      return(-1);
   }
	fs_p->nr_iods = ack.ack.iod_info.nr_iods;
	close(s);         /* closes socket with manager  */
	/* Prepare iod request     */
	iodreq.type  				= IOD_INSTR;
	iodreq.dsize				= 0;
	iodreq.req.instr.flags 	= 0;
	/* Send iod request   		*/	
	for (i=0; i < fs_p->nr_iods; i++) {
		if ((s = new_sock()) < 0) return(-1);
pvfs_instr_connect:
		if (connect(s, (struct sockaddr *)&(fs_p->iod[i].addr), sizeof(struct sockaddr_in)) < 0) {
			if (errno == EINTR) goto pvfs_instr_connect;
			PERROR("pvfs_instr: connect"); return(-1);
		}
		fs_p->iod[i].sock = s;
		if ((ret = bsend(fs_p->iod[i].sock, &iodreq, sizeof(ireq))) < 0) {
			/* error sending request */
			perror("pvfs_instr: bsend: ireq");
			return(-1);
		}
		if ((ret = brecv(fs_p->iod[i].sock, &(fs_p->iod[i].ack), 
			sizeof(iack))) < 0) {
			/* error receiving ack */
			perror("pvfs_instr: brecv:");
			return(-1);
		}
	}	
	/* Check acks  */
	for (i=0; i < fs_p->nr_iods; i++) {
		if (fs_p->iod[i].ack.status) {
			errno = fs_p->iod[i].ack.eno;
			for (i=0;i<fs_p->nr_iods;i++) close(fs_p->iod[i].sock);
			return(fs_p->iod[i].ack.status);
		}
	}
	for (i = 0; i < fs_p->nr_iods; i++) {
		printf("IOD %d:\n", i);
		printf("Average access time: %lf\n",
			fs_p->iod[i].ack.ack.instr.avg);
		printf("Average wait time: %lf\n",
			fs_p->iod[i].ack.ack.instr.wait);
		printf("Number of accesses: %d\n", fs_p->iod[i].ack.ack.instr.accesses);	
		close(fs_p->iod[i].sock);
	}		
	return(0);
}
