/* pvfs_lstat.c copyright (c) 1996 Clemson University, all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_lstat.c,v $
 * Revision 1.8  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.7  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.5  1998/02/21 03:29:16  rbross
 * yet another stat fix
 *
 * Revision 1.4  1998/02/19 23:03:06  rbross
 * removed SYS_stat defines ... moving to lib.h
 *
 * Revision 1.3  1998/02/16 22:03:56  rbross
 * repatched to work with libc5 too
 *
 * Revision 1.2  1998/02/16 19:06:07  rbross
 * Newt patches
 *
 * Revision 1.1  1997/11/17 21:08:30  walt
 * Initial revision
 *
 *
/*/

static char pvfs_lstat_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_lstat.c,v 1.8 1998/07/10 20:20:00 rbross Exp $";

/* This file contains PVFS library call for lstat.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>
#include <sys/stat.h>

extern int errno;

int pvfs_lstat(char* pathname, struct stat *buf)
{
	int i, s;
	char host[256];
	mack ack;
	mreq request;
	char *t_p;
	dmeta dir;
	int retval;

	retval = unix_lstat(pathname, buf);
	if (S_ISLNK(buf->st_mode)) return retval;

	t_p = pathname + strlen(pathname);
	while (*t_p != '/' && t_p > pathname) t_p--;
	
	/* check if PVFS file */
	if ((i = check_pvfs(pathname)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return retval;
	}
	if ((i = get_dmeta(pathname, &dir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	*(dir.sd_path-1) = '/';
	*(dir.fname-1) = '/';
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_STAT;
	request.dsize = strlen(dir.rd_path);
	request.req.stat.fname = 0;

	/* Send request to mgr */
	if ((s = send_mreq_dmeta(&dir, &request, dir.rd_path, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	*buf = ack.ack.stat.meta.u_stat;
	return (0);
}
