/* pvfs_mount.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_mount.c,v $
 * Revision 1.11  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.10  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.8  1998/02/16 22:03:56  rbross
 * repatched to work with libc5 too
 *
 * Revision 1.7  1997/10/27 21:19:14  pvfs
 * added license
 *
 * Revision 1.6  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.5  1997/01/20 17:17:28  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1997/01/16 19:35:29  mcettei
 * uses debugging code
 *
 * Revision 1.3  1996/12/06 20:51:04  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1996/12/06 20:48:27  mcettei
 * added header
 *
 *
/*/

static char pvfs_mount_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_mount.c,v 1.11 1998/07/10 20:20:00 rbross Exp $";

/* This file contains PVFS library call for mount.   		*/
#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_mount(const char* specialfile, char* dir, const char*
	filesystemtype, unsigned long rwflag, const void* data)
{
	fmeta meta1;
	int fd, s, i, flag, mode, len; 
	char host[256];
	mreq req;
	mack ack;
	dmeta dotdir;

	len = strlen(dir);
	if (dir[len-1] != '/') dir[len] = '/';
	/* Check to see if .pvfsdir exists in directory  	*/
	if ((i = check_pvfs(dir)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return (syscall(SYS_mount, specialfile, dir, filesystemtype,
				rwflag, data));
	}
	if ((i = get_dmeta(dir, &dotdir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}
	/* Prepare request for file system  */
	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_MOUNT;
	req.dsize = strlen(dotdir.rd_path);
	req.req.mount.fsname = 0;

	/* send request to manager */
	if ((s = send_mreq_dmeta(&dotdir, &req, dotdir.rd_path, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return (0);
}
