/* pvfs_open.c copyright (c) 1996 R. Ross and M. Cettei, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_open.c,v $
 * Revision 1.33  1999/06/17 15:36:46  rbross
 * added O_CONN support
 *
 * Revision 1.32  1999/02/18 19:31:40  rbross
 * added sanity checking for fdesc and iod_info structures
 *
 * Revision 1.31  1998/07/23 21:54:06  rbross
 * open support
 *
 * Revision 1.30  1998/07/21 04:16:09  rbross
 * *** empty log message ***
 *
 * Revision 1.29  1998/07/14 18:40:47  rbross
 * *** empty log message ***
 *
 * Revision 1.27  1998/07/08 20:09:19  rbross
 * cleaned up
 *
 * Revision 1.24  1998/07/02 18:35:45  rbross
 * modified to use new check_pvfs() return values
 *
 * Revision 1.23  1998/02/16 19:12:10  rbross
 * Newt patches
 *
 * Revision 1.22  1997/12/03 21:28:09  pvfs
 * fixed O_APPEND process
 *
 * Revision 1.21  1997/11/24 23:00:11  rbross
 * *** empty log message ***
 *
 * Revision 1.20  1997/10/27 21:19:24  pvfs
 * added license
 *
 * Revision 1.19  1997/07/10 18:36:51  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.18  1997/07/01 20:25:45  mcettei
 * improved default values for pvfs_stat
 *
 * Revision 1.17  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.16  1997/06/04 15:27:03  mcettei
 * *** empty log message ***
 *
 * Revision 1.15  1997/03/22 22:33:59  rbross
 * works with group connections
 *
 * Revision 1.14  1997/03/18 00:08:19  rbross
 * modified to support retries on passive group connections
 *
 * Revision 1.13  1997/03/13 18:32:04  mcettei
 * changed pvfs_creat to behave like unix creat
 *
 * Revision 1.12  1997/03/13 18:23:38  rbross
 * *** empty log message ***
 *
 * Revision 1.11  1997/03/13 04:17:49  mcettei
 * *** empty log message ***
 *
 * Revision 1.10  1997/02/26 20:47:21  rbross
 * corrected initialization problems
 *
 * Revision 1.9  1997/02/24 19:44:52  rbross
 * *** empty log message ***
 *
 * Revision 1.8  1997/01/22 21:40:30  mcettei
 * add pvfs_creat and send_open_req functions
 *
 * Revision 1.7  1997/01/20 17:18:24  mcettei
 * *** empty log message ***
 *
 * Revision 1.6  1997/01/14 21:52:15  rbross
 * working (?) version
 *
 * Revision 1.5  1997/01/08 16:58:49  mcettei
 * removed getsockname, done in mgr.c
 *
 * Revision 1.4  1997/01/07 18:29:27  mcettei
 * will save manager sockaddr in meta structure
 *
 * Revision 1.3  1996/12/06 20:50:33  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1996/12/06 20:49:22  mcettei
 * added header
 *
 *
/*/

static char pvfs_open_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_open.c,v 1.33 1999/06/17 15:36:46 rbross Exp $";

/* This file contains PVFS library call for open.   		*/
/* Should work to create and open files 						*/

#include <lib.h>
#include <signal.h>

extern int errno;
extern sockset socks;
extern jlist_p active_p;


fdesc_p pfds[NR_OPEN] = {NULL};
static int send_open_req(const char*, dmeta_p, mreq_p);
int unix_open(const char *, int, mode_t, fpart_p);

/* pvfs_open() - PVFS open command.
 * pathname - file to be opened
 * flag - flags (O_RDONLY, etc.) to be used in open
 * mode - if creating file, mode to be used
 * ptr1, ptr2 - these point to metadata and partition information,
 *    if provided.  The flags (O_META and O_PART) must be set if one
 *    is to be used, and if both metadata and partition information
 *    are provided then the metadata pointer MUST PRECEDE the partition
 *    information.
 */
int pvfs_open(const char* pathname, int flag, mode_t mode,
	void *ptr1, void *ptr2)
{
	int 		i, fd;
	char		*wd, *path;
	fdesc_p 	newfile;
	ino_t   	fs_ino;
	mreq 		req;
	pvfs_stat_p meta_p;
	fpart_p     part_p;
	dmeta		dir;

	atexit((void *)pvfs_exit);
	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}
	/* Check to see if file is a PVFS file */
	if ((i = check_pvfs(pathname)) != 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		if (flag & O_PART) {
			if (flag & O_META) part_p = ptr2;
			else part_p = ptr1;
		}
		else part_p = NULL;
		return(unix_open(pathname, flag, mode, part_p));
	}
	if (get_dmeta(pathname, &dir) < 0) {
		PERROR("Error getting dotfile data");
		return(-1);
	}

	/* Prepare request for file system */
	if (flag & O_META) {
		meta_p = ptr1;
		req.req.open.meta.p_stat.base = meta_p->base;
		req.req.open.meta.p_stat.pcount = meta_p->pcount;
		req.req.open.meta.p_stat.ssize = meta_p->ssize;
		req.req.open.meta.p_stat.soff  = meta_p->soff;
		req.req.open.meta.p_stat.bsize = meta_p->bsize;
	}
	else {
		req.req.open.meta.p_stat.base = -1;
		req.req.open.meta.p_stat.pcount = -1;
		req.req.open.meta.p_stat.ssize = -1;
		req.req.open.meta.p_stat.soff  = 0;
		req.req.open.meta.p_stat.bsize = -1;
	}

	req.uid = getuid();
	req.gid = getgid();
	req.type = MGR_OPEN;

	/* BOY IS THIS A HACK...WHO DID THIS ONE? -- ROB */
	*(dir.sd_path-1) = '/'; /* form one lone path name */
	*(dir.fname-1) = '/';
	req.dsize = strlen(dir.rd_path);
	req.req.open.fname = 0;
	req.req.open.flag = flag;
	req.req.open.mode  = mode;
	req.req.open.meta.fs_ino = dir.fs_ino;
	req.req.open.meta.u_stat.st_uid = req.uid;
	req.req.open.meta.u_stat.st_gid = req.gid;
	req.req.open.meta.u_stat.st_mode = mode;

	if (flag & O_APPEND) 
		req.req.open.flag = flag & ~(O_APPEND);

	if ((fd = send_open_req(pathname, &dir, &req)) < 0) return(-1);

	/* set partition (if provided) */
	if (flag & O_PART) {
		if (flag & O_META) part_p = ptr2;
		else part_p = ptr1;
		
		if (!(pfds[fd]->part_p = malloc(sizeof(fpart)))) {
			PERROR("pvfs_open: malloc");
			return(-1);
		}
		*(pfds[fd]->part_p) = *part_p;
	}

	/* open all IOD connections (if requested) */
	if (flag & O_CONN) {
		ireq iodreq;

		iodreq.majik_nr   = IOD_MAJIK_NR;
		iodreq.type       = IOD_NOOP;
		iodreq.dsize      = 0;

		/* build job to send reqs and recv acks to/from iods */
		if (!active_p) active_p = jlist_new();
		initset(&socks); /* clear out the socket set */

		if (build_simple_jobs(pfds[fd], &iodreq) < 0) {
			ERR("pvfs_open: build_simple_jobs failed...continuing\n");
			return(fd);
		}

		/* call do_job */
		while (!jlist_empty(active_p)) {
			if (do_jobs(active_p, &socks, -1) < 0) {
				ERR("pvfs_open: do_jobs failed...continuing\n");
				return(fd); /* we'll let it slide for now */
			}
		}
		/* don't bother looking for errors */
	}

	/* move to end of file (if necessary) */
	if (flag & O_APPEND) {
		pvfs_lseek(fd, 0, SEEK_END);
	}
	return(fd);
}

int pvfs_creat(const char* pathname, mode_t mode) {
	int flags;
	
	/* call pvfs_open with correct flags            */
	flags  =  O_CREAT|O_WRONLY|O_TRUNC;
	pvfs_open(pathname, flags, mode, NULL, NULL); 

}

/* send_open_req() - sends request to the manager, sets up FD structure,
 * returns file descriptor number
 *
 * Returns -1 on error.
 */
static int send_open_req(const char* pathname, dmeta_p dir_p, mreq_p req_p) {
	char send_req = 10;
	int fd, s, i;
	mack ack;

	/* Send request to mgr */
	for (;;) {
		s = send_mreq_dmeta(dir_p, req_p, dir_p->rd_path, &ack);
		/* Check ack */
		if (s >= 0 && !ack.status) break; /* went through */

		/* problems... */
		errno = ack.eno;
		if ((req_p->req.open.flag & O_GADD) && errno == EAGAIN && send_req) {
			LOG("received try again from manager\n");
			close(s);
			sleep(2);
			send_req--;
		}
		else {
			return(-1);
		}	
	}
	if ((fd = syscall(SYS_open, pathname, O_RDONLY, 0)) < 0) {
		int myerr = errno;
		if (myerr == ENOENT) {
			/* special case, we know that the file SHOULD be there, but it
			 * isn't yet...this is probably because NFS directory info
			 * isn't up to date yet.  We're going to open /dev/null
			 * instead.  I don't THINK we use this FD anywhere else, so it
			 * should be ok, but I can't remember.
			 *
			 * -- Rob
			 */
			if ((fd = syscall(SYS_open, "/dev/null", O_RDONLY, 0)) < 0) {
				myerr = errno;
				PERROR("pvfs_open: error opening /dev/null instead of metafile");
				errno = myerr;
				return(-1);
			}
		}
		else {
			PERROR("pvfs_open: error opening metafile");
			errno = myerr;
			return(-1);
		}
	}	
	LOG1("Opening: %s\n", pathname);
	pfds[fd] = (fdesc_p)malloc(sizeof(fdesc) + sizeof(iod_info) * 
		(ack.ack.open.meta.p_stat.pcount - 1));
#ifdef STRICT_DESC_CHECK
	pfds[fd]->checkval  = FDESC_CHECKVAL;
#endif
	pfds[fd]->fs        = FS_PVFS; /* PVFS file type */
	pfds[fd]->fd.off    = 0;
	pfds[fd]->fd.grp_nr = -1;
	pfds[fd]->fd.flag   = req_p->req.open.flag;
	pfds[fd]->fd.meta   = ack.ack.open.meta;
	pfds[fd]->fd.cap    = ack.ack.open.cap;
	pfds[fd]->fd.ref    = 1;
	pfds[fd]->part_p    = NULL;
	pfds[fd]->dim_p     = NULL;

	/* Ought to initialize part_p and dim_p of fdesc */	
   if ((i = brecv(s, pfds[fd]->fd.iod, ack.dsize)) < 0) {
      PERROR("pvfs_open: brecv");
      return(-1);
   }
	for (i=0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
#ifdef STRICT_DESC_CHECK
		pfds[fd]->fd.iod[i].checkval = IOD_INFO_CHECKVAL;
#endif
		pfds[fd]->fd.iod[i].sock = -1;
	}
	
	/* if this is a passive (group) connection, should call pvfs_gadd */
	return(fd);
}

int unix_open(const char *pathname, int flag, mode_t mode, fpart_p part_p)
{
	int fd, myerr;
	pvfs_stat p_stat={0,1,8192,8192}; /* metadata */

	if ((fd = (syscall(SYS_open, pathname, flag & ~PVFSMASK, mode))) < 0) {
		return(fd);
	}
	if (!(pfds[fd] = (fdesc_p)malloc(sizeof(fdesc)))) {
		myerr = errno;
		close(fd);
		errno = myerr;
		return(-1);
	}
#ifdef STRICT_DESC_CHECK
	pfds[fd]->checkval = FDESC_CHECKVAL;
#endif
	pfds[fd]->fs = FS_UNIX;
	pfds[fd]->part_p    = NULL;
	pfds[fd]->dim_p     = NULL;
	pfds[fd]->fd.off    = 0;
	pfds[fd]->fd.grp_nr = -1;
	pfds[fd]->fd.flag   = flag;
	pfds[fd]->fd.cap    = 0;
	pfds[fd]->fd.ref    = 1;
	pfds[fd]->fd.meta.p_stat = p_stat; /* fill in some dummy values */
	/* set partition (if provided) */
	if (flag & O_PART) {
		if (!(pfds[fd]->part_p = malloc(sizeof(fpart)))) {
			myerr = errno;
			close(fd);
			errno = myerr;
			return(-1);
		}
		*(pfds[fd]->part_p) = *part_p;
	}
	return(fd);
}

#ifdef STRICT_DESC_CHECK
int do_fdesc_check(int fd)
{
	int i, err=0;
	if (fd < 0 || !pfds[fd]) {
		fprintf(stderr, "do_fdesc_check: invalid fd (fd = %d)\n", fd);
		return(-1);
	}
	if (pfds[fd]->checkval != FDESC_CHECKVAL) {
		fprintf(stderr, "do_fdesc_check: fdesc struct (fd = %d) trashed?\n", fd);
		return(-1);
	}
	if (pfds[fd]->fs != FS_PVFS) return(0);

	for (i=0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
		if (pfds[fd]->fd.iod[i].checkval != IOD_INFO_CHECKVAL) {
			err = -1;
			fprintf(stderr, "do_fdesc_check: iod_info struct"
				" (fd = %d, iod = %d) trashed?\n", fd, i);
		}
	}
	return(err);
}
#endif
