/* pvfs_ostat.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_ostat.c,v $
 * Revision 1.16  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.15  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.13  1998/02/21 03:29:16  rbross
 * yet another stat fix
 *
 * Revision 1.12  1998/02/19 23:03:06  rbross
 * removed SYS_stat defines ... moving to lib.h
 *
 * Revision 1.11  1998/02/16 22:03:56  rbross
 * repatched to work with libc5 too
 *
 * Revision 1.10  1998/02/16 19:13:38  rbross
 * Newt patches
 *
 * Revision 1.9  1997/10/27 21:19:35  pvfs
 * added license
 *
 * Revision 1.8  1997/06/27 17:23:02  walt
 * changed syscall to SYS_stat - the "proper" call name
 *
 * Revision 1.7  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.6  1997/01/29 19:59:16  rbross
 * fixed up to handle dot files in directory, etc. etc.
 *
 * Revision 1.5  1997/01/20 17:19:49  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1997/01/16 19:37:04  mcettei
 * uses debugging code
 *
 * Revision 1.3  1996/12/09 20:15:19  mcettei
 * return correct structure
 *
 * Revision 1.2  1996/12/05 18:53:17  mcettei
 * added header
 *
/*/

static char pvfs_ostat_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_ostat.c,v 1.16 1998/07/10 20:20:00 rbross Exp $";

/* This file contains PVFS library call for stat.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_ostat(char* pathname, struct stat *buf)
{
	int i, s;
	int succ;
	char host[256];
	mack ack;
	mreq request;
	char *t_p;
	dmeta dir;

	if (!pathname) {
		errno = EFAULT;
		return(-1);
	}

	/* Check to see if .pvfsdir exists in directory  	*/
	/* also check to see if they're listing a dot file */
	t_p = pathname + strlen(pathname);
	while (*t_p != '/' && t_p > pathname) t_p--;
	
	if ((i = check_pvfs(pathname)) < 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return (unix_stat(pathname, buf));
	}
	if ((i = get_dmeta(pathname, &dir)) < 0) {
		PERROR("Error getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	*(dir.sd_path-1) = '/';
	*(dir.fname-1) = '/';
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_STAT;
	request.dsize = strlen(dir.rd_path);
	request.req.stat.fname = 0;

	/* Send request to mgr */
	if ((s = send_mreq_dmeta(&dir, &request, dir.rd_path, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	*buf = ack.ack.stat.meta.u_stat;
	return (0);
}
