/* pvfs_rename.c copyright (c) 1996 Rob Ross and Matt Cettei,
   all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_rename.c,v $
 * Revision 1.10  1999/03/01 22:19:18  rbross
 * Maciek's suggested fixes
 *
 * Revision 1.9  1998/08/18 18:09:32  spmcmil
 * Modified to work with . and .. directory modifiers properly.
 *
 * Revision 1.8  1998/08/07 17:16:03  spmcmil
 * Modified to pass full pathnames of files to manager.  Also had to add system
 * call to catch the rename call.
 *
 * Revision 1.7  1998/07/10 20:20:00  rbross
 * updates, some including new get_dmeta() usage
 *
 * Revision 1.6  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.4  1998/02/16 19:14:44  rbross
 * Newt patches
 *
 * Revision 1.3  1997/10/27 21:22:12  pvfs
 * added license
 *
 * Revision 1.2  1997/06/10 17:56:55  walt
 * updated to handle directories
 *
 * Revision 1.1  1997/02/10 19:09:17  pvfs
 * Initial revision
 *
/*/

static char pvfs_unlink_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_rename.c,v 1.10 1999/03/01 22:19:18 rbross Exp $";

#include <sys/param.h>
#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_rename(const char *oldname, const char *newname)
{
	int j,i, s, index;
	mreq request;
	mack ack;
	char d_p[MAXPATHLEN];
	dmeta odir,ndir;

	if (!oldname || !newname) {
		errno = EFAULT;
		return(-1);
	}

	/* check to see if file is a pvfs file */
	i = check_pvfs(newname);
	j = check_pvfs(oldname);
	if (i==0 && j==0) {
		/* nonpvfs stuff */
		return(syscall(SYS_rename, oldname, newname));
	}
	if (i< 0 || j < 0) {
		/* oldname or newname doesn't exist */
		errno = ENOENT;
		return(-1);
	}
	if (i == 0 || j == 0) {
		/* across file systems, error */
		errno = EXDEV; /* not on same file system! */
		return(-1);
	}

	/* setup full pathnames for pvfs files */
	if (oldname[0] != '/') {
		/* get dmeta for oldname stuff */
		if (get_dmeta(oldname, &odir) < 0) {
			PERROR("pvfs_rename: get_dmeta");
			return(-1);
		}

		/* okay, absolute pathname not specified */
		strncpy(d_p,odir.rd_path,MAXPATHLEN);
		strncat(d_p,"/",MAXPATHLEN);
		strncat(d_p,odir.sd_path,MAXPATHLEN);
		strncat(d_p,"/",MAXPATHLEN);
		/* get_dmeta takes care of stripping the .. and . modifiers */
		strncat(d_p,odir.fname,MAXPATHLEN);
	}
	else
		strncpy(d_p,newname,MAXPATHLEN);

	/* add new filename path to request */
	if (newname[0] != '/') {
		/* newname dmeta stuff */
		if (get_dmeta(newname, &ndir) < 0) {
			PERROR("pvfs_rename: get_dmeta");
			return(-1);
		}

		/* absolute path not specified */
		index = strlen(d_p);
		strncat(d_p,"*",MAXPATHLEN);
		strncat(d_p,ndir.rd_path,MAXPATHLEN);
		strncat(d_p,"/",MAXPATHLEN);
		strncat(d_p,ndir.sd_path,MAXPATHLEN);
		strncat(d_p,"/",MAXPATHLEN);
		/* get_dmeta takes care of stripping the .. and . modifiers */
		strncat(d_p,ndir.fname,MAXPATHLEN);
	}
	else
		strncat(d_p,newname,MAXPATHLEN);
	
	/* Prepare request for file system  */
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_RENAME;
	request.dsize = strlen(d_p);
	request.req.rename.oname = 0;
	request.req.rename.nname = 0;

	d_p[index] = '\0';
	/* Send request to mgr */	
	/* both old and new names better beon the same host and port number */
	if ((ndir.host != odir.host) || (ndir.port != odir.port)) {
		errno = EXDEV; /* not on same file system! */
		PERROR("pvfs_rename:");
		return(-1);
	}
	if ((s = send_mreq_dmeta(&ndir, &request, d_p, &ack)) < 0) {
		PERROR("send_mreq");
		return(-1);
	}
	return(0);
}
