/* pvfs_statfs.c copyright (c) 1999 Clemson University, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
/*/

/*
 * $Log$
 *
/*/

static char pvfs_statfs_c[] = "$Header$";

/* This file contains PVFS library call for fchmod.  		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/

#include <lib.h>
#include <meta.h>
#include <sys/vfs.h>
#include <linux/ext2_fs.h>

extern int errno;
extern fdesc_p pfds[];

int pvfs_fstatfs(int fd, struct statfs *buf)
{
	/* check for badness */
	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  
	/* check for UNIX */
	if (!pfds[fd] || pfds[fd]->fs==FS_UNIX) {
		return(syscall(SYS_fstatfs, fd, buf));
	}

	buf->f_type = PVFS_SUPER_MAGIC;
	buf->f_bsize = pfds[fd]->fd.meta.p_stat.ssize; /* return the stripe size */
	buf->f_blocks = -1;
	buf->f_bfree = -1;
	buf->f_bavail = -1;
	buf->f_files = -1;
	buf->f_ffree = -1;
	/* don't know what to do with fsid...leaving it be... */
	buf->f_namelen = EXT2_NAME_LEN;
	buf->f_spare[0] = -1;

	return(0);
}

int pvfs_statfs(char *path, struct statfs *buf)
{
	int i, ret, myerr;
	if ((i = pvfs_open(path, O_RDONLY)) < 0) return(-1);
	ret = pvfs_fstatfs(i, buf);
	myerr = errno;
	pvfs_close(i);
	errno = myerr;
	return(ret);
}
