/*
	pvfs_truncate.c copyright (c) 1998 Clemson University
   all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Walt Ligon walt@eng.clemson.edu

/*/

/*
 * $Log: pvfs_truncate.c,v $
 * Revision 1.1  1998/07/23 21:54:06  rbross
 * Initial revision
 *
 *
/*/

static char pvfs_truncate_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_truncate.c,v 1.1 1998/07/23 21:54:06 rbross Exp $";

#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_truncate(const char *path, size_t length)
{
	int i, s;
	mreq request;
	mack ack;
	char *d_p;
	dmeta dir;

	if (!path) {
		errno = EFAULT;
		return(-1);
	}

	/* check to see if file is a pvfs file */
	if ((i = check_pvfs(path)) != 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return(syscall(SYS_truncate, path, length));
	}
	if (get_dmeta(path, &dir) < 0) {
		PERROR("Error getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_TRUNCATE;
	request.dsize = strlen(path) + 1;
	request.req.truncate.fname = 0;
	request.req.truncate.length = length;

	if ((d_p = malloc(request.dsize)) == NULL) {
		PERROR("malloc");
		return(-1);
	}
	strcpy(d_p, path);
	d_p[request.dsize] = '\0';


	/* Send request to mgr */	
	if ((s = send_mreq_dmeta(&dir, &request, d_p, &ack)) < 0) {
		PERROR("send_mreq");
		free(d_p);
		return(-1);
	}
	free(d_p);
	return(0);
}
