/* pvfs_utime.c copyright (c) 1996 Rob Ross and Matt Cettei,
   all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
 * $Log: pvfs_utime.c,v $
 * Revision 1.2  1998/08/18 18:08:59  spmcmil
 * Now a working version.
 *
 * Revision 1.1  1998/08/07 17:16:59  spmcmil
 * Initial revision
 *
/*/

static char pvfs_utime_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_utime.c,v 1.2 1998/08/18 18:08:59 spmcmil Exp $";

#include <utime.h>
#include <sys/types.h>
#include <sys/param.h>
#include <syscall.h>
#include <lib.h>

extern int errno;

int pvfs_utime(const char *filename, struct utimbuf *buf)
{
	dmeta dir;
	mreq request;
	mack ack;
	char d_p[MAXPATHLEN]; /* send buffer */

	/* check if this file is pvfs */
	if (check_pvfs(filename) < 1)
		return(syscall(SYS_utime, filename, buf));

	/* setup full pathnames for pvfs files */
	if (filename[0] != '/') {
		/* get dmeta to help build utime request */
		if (get_dmeta(filename, &dir) < 0) {
			PERROR("pvfs_utime: get_dmeta");
			return(-1);
		}

		/* okay, absolute pathname not specified */
      strncpy(d_p,dir.rd_path,MAXPATHLEN);
      strncat(d_p,"/",MAXPATHLEN);
      strncat(d_p,dir.sd_path,MAXPATHLEN);
      strncat(d_p,"/",MAXPATHLEN);
		/* get_dmeta strips .. and . modifiers for us */
      strncat(d_p,dir.fname,MAXPATHLEN);
	}
	else
		strncpy(d_p,filename,MAXPATHLEN);

	/* prepare request for manager */
   request.uid = getuid();
   request.gid = getgid();
   request.type = MGR_UTIME;
   request.dsize = strlen(d_p);
   request.req.utime.fname = 0;
	/* if buf is null, set to current time */
	if (buf) {
		request.req.utime.buf.actime = buf->actime; /* access time */
		request.req.utime.buf.modtime = buf->modtime; /* modification time */
	}
	else {
		request.req.utime.buf.actime = time(NULL);
		request.req.utime.buf.modtime = request.req.utime.buf.actime;
	}
		
	/* success, I guess */
   /* Send request to mgr */
   if (send_mreq_dmeta(&dir, &request, d_p, &ack) < 0) {
      PERROR("send_mreq");
      return(-1);
   }

	return(0);
}
