
/*
/* flist.c copyright (c) 1996 Clemson University, all rights reserved.

	Written by Rob Ross and Matt Cettei.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* FLIST.C - functions to handle the creation and modification of 
/* lists of file information
/*
/* Note: a SINGLE entry is added to the list for an open file, 
/* no matter how many times the file has been opened.
/*
/* This is built on top of the llist.[ch] files
/*
/*/

/*
 * $Log: flist.c,v $
 * Revision 1.13  1999/06/17 15:40:21  rbross
 * *** empty log message ***
 *
 * Revision 1.12  1998/07/21 04:20:21  rbross
 * f_rem() no longer static
 *
 * Revision 1.11  1998/07/05 01:54:01  rbross
 * updated to include forall_finfo()
 *
 * Revision 1.10  1998/02/16 19:19:08  rbross
 * Newt patches
 *
 * Revision 1.9  1997/11/11 19:05:30  rbross
 * changed f_new() to initialize unlinked value to -1 (now holds fd)
 *
 * Revision 1.8  1997/11/11 15:43:00  rbross
 * fixed dumb bug
 *
 * Revision 1.7  1997/11/11 15:40:27  rbross
 * added f_new() for allocating new finfo structures
 *
 * Revision 1.6  1997/10/28 18:53:20  pvfs
 * added license
 *
 * Revision 1.5  1997/07/10 18:40:26  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.4  1997/03/22 22:35:28  rbross
 * *** empty log message ***
 *
 * Revision 1.3  1997/03/18 00:17:16  rbross
 * *** empty log message ***
 *
 * Revision 1.2  1997/01/20 22:30:46  rbross
 * added flist_empty call
 *
 * Revision 1.1  1997/01/20 18:12:02  rbross
 * Initial revision
 *
/*/

#include <stdlib.h>
#include <sys/types.h>
#include <flist.h>

static char header[] = "$Header: /projects/pvfs/pvfs-working/mgr/RCS/flist.c,v 1.13 1999/06/17 15:40:21 rbross Exp $";

void f_free(void *f_p);

/* prototypes for internal functions */
static int f_ino_cmp(void *key, void *f_p);

flist_p flist_new(void)
{
	return(llist_new());
}

int flist_empty(flist_p fl_p)
{
	return(llist_empty(fl_p));
}

int f_add(flist_p fl_p, finfo_p f_p)
{
	return(llist_add(fl_p, (void *) f_p));
}

finfo_p f_search(flist_p fl_p, ino_t f_ino)
{
	return((finfo_p) llist_search(fl_p, (void *) f_ino, f_ino_cmp));
}

int f_rem(flist_p fl_p, ino_t f_ino)
{
	finfo_p f_p;
	
	f_p = (finfo_p) llist_rem(fl_p, (void *) f_ino, f_ino_cmp);
	if (f_p) {
		f_free(f_p->f_name);
		f_free(f_p);
		return(0);
	}
	return(-1);
}

void flist_cleanup(flist_p fl_p)
{
	llist_free(fl_p, f_free);
}

static int f_ino_cmp(void *key, void *f_p)
{
	if ((ino_t)key == ((finfo_p)f_p)->f_ino) return(0);
	else return(1);
}

finfo_p f_new(void) {
	finfo_p f_p;

	if (!(f_p = (finfo_p) malloc(sizeof(finfo)))) return(0);

	f_p->cap = 0;
	f_p->cnt = 1;
	f_p->grp_cap = -1;
	f_p->f_name = 0;
	f_p->unlinked = -1;
	FD_ZERO(&f_p->socks);
	return(f_p);
}

void f_free(void *f_p)
{
	free((finfo_p)f_p);
}

int flist_dump(flist_p fl_p)
{
	return(llist_doall(fl_p, f_dump));
}

int f_dump(void *v_p)
{
	finfo_p f_p = (finfo_p) v_p; /* quick cast */

#ifdef DEBUG
	fprintf(stderr, "inode: %d\nbase: %d\npcount: %d\nssize: %d\n"
		"bsize: %d\ncap: %d\ncnt: %d\ngrp_cap: %d\n"
		"f_name: %s\n\n", f_p->f_ino,
		f_p->p_stat.base,
		f_p->p_stat.pcount, f_p->p_stat.ssize, f_p->p_stat.bsize,
		f_p->cap, f_p->cnt, f_p->grp_cap, f_p->f_name);
#endif
}

int forall_finfo(flist_p fl_p, int (*fn)(void *))
{
	return(llist_doall(fl_p, fn));
}
