/* flist.h copyright (c) 1996,1997 Clemson University, all rights reserved.

	Written by Rob Ross, Matt Cettei, and Walt Ligon.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* FLIST.H - header for flist.c, the manager file list manipulation
/* routines
/*/

/*
 * $Log: flist.h,v $
 * Revision 1.8  1998/07/05 01:54:01  rbross
 * updated to include forall_finfo()
 *
 * Revision 1.7  1997/11/11 19:05:13  rbross
 * changed unlinked type to integer so that it can hold fd value
 *
 * Revision 1.6  1997/11/11 15:40:02  rbross
 * added unlinked element to struct finfo
 *
 * Revision 1.5  1997/10/28 18:53:28  pvfs
 * added license
 *
 * Revision 1.4  1997/07/24 16:35:04  walt
 * added support for fchown and fchmod
 *
 * Revision 1.3  1997/03/18 00:16:19  rbross
 * added grp_cap to flist structure to keep up with group capability #;
 * only one group can exist per file at the moment
 *
 * Revision 1.2  1997/01/20 22:30:57  rbross
 * added prototype for flist_empty call
 *
 * Revision 1.1  1997/01/20 18:12:12  rbross
 * Initial revision
 *
/*/

#ifndef FLIST_H
#define FLIST_H

#include <llist.h>
#include <meta.h>
#include <sys/stat.h>
#include <unistd.h>

static char flist_h[] = "$Header: /projects/pvfs/pvfs-working/mgr/RCS/flist.h,v 1.8 1998/07/05 01:54:01 rbross Exp $";

typedef llist_p flist_p;

typedef struct finfo finfo, *finfo_p;

struct finfo {
	int unlinked;     /* fd of metadata file or -1 */
	ino_t f_ino;      /* inode # of metadata file */
	pvfs_stat p_stat; /* PVFS metadata for file */
	int cap;          /* max. capability assigned thusfar? */
	int cnt;          /* count of # of times file is open */
	int grp_cap;      /* capability # for group; this should be expanded
	                   * sooner or later so that multiple groups can
	                   * exist for a single file */
	char *f_name;     /* file name - for performing operations on metadata */
	fd_set socks;     /* used to keep track of what FDs have opened the file */
};

flist_p flist_new(void);
int flist_empty(flist_p);
int f_add(flist_p, finfo_p);
finfo_p f_search(flist_p, ino_t);
finfo_p f_new(void);
int f_rem(flist_p, ino_t);
void flist_cleanup(flist_p);
int f_dump(void *);
int forall_finfo(flist_p, int (*fn)(void *));

#endif
