/*
/* check_pvfs.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* CHECK_PVFS.C -  PVFSMGR version...
/*
/*/

/*
 * $Log: check_pvfs.c,v $
 * Revision 1.8  1998/07/02 18:32:56  rbross
 * modified to return 2 for PVFS directories
 *
 * Revision 1.7  1998/02/16 20:28:30  rbross
 * repatched to work with libc5
 *
 * Revision 1.6  1998/02/16 19:20:31  rbross
 * Newt patches
 *
 * Revision 1.5  1997/10/28 19:00:08  pvfs
 * added license
 *
 * Revision 1.4  1997/07/07 16:59:51  walt
 * fixed numerous bugs needed to get libc running
 *
 * Revision 1.3  1997/06/10 18:13:18  walt
 * modified to handle directories correctly
 *
 * Revision 1.2  1997/06/10 17:52:18  walt
 * updated directory handling code
 *
 * Revision 1.1  1997/01/16 16:30:50  mcettei
 * Initial revision
 *
 *
/*/

static char check_pvfs_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/meta/RCS/check_pvfs.c,v 1.8 1998/07/02 18:32:56 rbross Exp $";
/* These functions will determine if a given file is a pvfs file	*/
/* Returns 1 if pvfs, 0 if possibly UNIX, -1 if neither				*/

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/timeb.h>
#include <linux/types.h>
#include <syscall.h>
#include <fcntl.h>
#include <unistd.h>
#include <meta.h>
#include <req.h>

#ifndef SYS_stat
#define SYS_stat (SYS_prev_stat)
#endif

extern int errno;

/*
   This code checks to see if the file pointed to by pathname is a PVFS file
   we decide this as follows: fo regular files, if there is a file named
   .pvfsdir in the directory with the file, then it is a PVFS file, for
   directories, if there is a file named .pvfsdir in the directory, then it
   is a PVFS directory.  We have to be prepared for almost anything to be passed
   in via pathname - thus all of the convoluted string editing below
*/

#define MAXLEN 256
int check_pvfs(const char* pathname)
{
	fmeta meta1;
	int isdir=0, length;
	char temp[MAXLEN];
	struct stat foo;

   strncpy(temp, pathname, MAXLEN);
   length = strlen(temp);
   if (length == 0) {
		/* this should never happen - WBL */
   	strncat(temp, ".", MAXLEN);
   }
   if (syscall(SYS_stat, temp, &foo) < 0) {
		/* file doesn't exist - need to fool S_ISDIR */
		foo.st_mode = 0;
   }
   if (!S_ISDIR(foo.st_mode)) {
		for (; length >= 0 && temp[length] != '/'; temp[length--] = 0);
   }
   else {
		/* this is a directory */
		isdir=1;
   	strncat(temp, "/", MAXLEN);
   }
   strncat(temp, ".pvfsdir", MAXLEN);
   if (syscall(SYS_stat, temp, &foo) < 0) {
		return(0);
   }
   else {
		/* Otherwise must be pvfs file */
		return(1+isdir);
   }
}	
