
/*
/* md_chmod.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* MD_CHMOD.C - 
/*
/*/

/*
 * $Log: md_chmod.c,v $
 * Revision 1.9  1998/08/07 17:28:09  spmcmil
 * Modified meta_access stuff and cleaned up (hopefully)
 *
 * Revision 1.7  1998/07/20 17:31:44  spmcmil
 * Modified to use meta functions (meta_write/read/creat/open/close/unlink).
 *
 * Revision 1.6  1998/02/16 19:24:57  rbross
 * Newt patches
 *
 * Revision 1.5  1997/10/28 19:16:06  pvfs
 * added license
 *
 * Revision 1.4  1997/06/10 17:52:18  walt
 * updated directory handling code
 *
 * Revision 1.3  1997/06/09 19:28:27  walt
 * added directory support
 *
 * Revision 1.2  1997/01/30 20:21:28  rbross
 * fixed odd "?" in GNU ls output by ORing mode w/0100000
 *
 * Revision 1.1  1997/01/16 16:35:47  mcettei
 * Initial revision
 *
 *
/*/

static char md_chmod_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/meta/RCS/md_chmod.c,v 1.9 1998/08/07 17:28:09 spmcmil Exp $";
/*This file contains the chmod metadata file call for the PVFS*/

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#include <linux/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <meta.h>
#include <req.h>

extern int errno;

int md_chmod(request)
mreq_p request;
{
fmeta meta1;
int fd,i,mask,length;
char temp[MAXPATHLEN];
dmeta dir;

	/* Open file listed in request */
   if ((fd = meta_open(request->req.chmod.fname, O_RDWR)) < 0) {
		if ((int)errno != (int)EISDIR) {
			perror("md_chmod: meta_open");
			meta_close(fd);
			return (-1);
		}
	}

	/* check execute permissions on directory */
	strncpy(temp, request->req.open.fname, MAXPATHLEN);
	length = get_parent(temp);
	/* length < 0 means CWD */
	if (length >= 0) {
		if (meta_access(0, request->req.chmod.fname, request->uid,
			request->gid, X_OK) < 0) {
			perror("md_chmod: meta_access");
			return(-1);
		}
	}
	
	/* Read metadata file */	
	if (fd >= 0) {
		if ((i = meta_read(fd, &meta1)) != sizeof(fmeta)) {
			perror("md_chmod: meta_read");
			meta_close(fd);
			return (-1);
		}
		i = lseek(fd, 0, SEEK_SET);
	}
	else { /* directory */
		get_dmeta(request->req.stat.fname, &dir);
		meta1.u_stat.st_uid = dir.dr_uid;
	}
	/* Change  if necessary */
	if ((request->uid != 0)&&(request->uid != meta1.u_stat.st_uid)) {
		errno = EPERM;
		perror("md_chmod");
		if (fd >= 0)
			meta_close(fd);
		return(-1);
	}

	/* had to add OR to get the high bits set right */
	meta1.u_stat.st_mode = request->req.chmod.mode | S_IFREG;

	/* Write metadata back */
	if (fd >= 0) {
		if ((i = meta_write(fd, &meta1, sizeof(fmeta))) != sizeof(fmeta)) {
			perror("md_chmod: meta_write");
			meta_close(fd);
			return(-1);
		}
	}
	else { /* directory */
		dir.dr_mode = (meta1.u_stat.st_mode & ~S_IFREG) | S_IFDIR;
		put_dmeta(request->req.chmod.fname, &dir);
	}
	
	/* Close metadata file */
	if (fd >= 0 && (i = meta_close(fd)) < 0) {
		perror("md_chmod: meta_close");
		return(-1);
	}

	/* Do acknowledge and return */
	return (0);
}
