/*
/* md_chown.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* MD_CHOWN.C - 
/*
/*/

/*
 * $Log: md_chown.c,v $
 * Revision 1.8  1998/08/07 17:28:34  spmcmil
 * Modified meta_access stuff and cleaned up.
 *
 * Revision 1.6  1998/07/20 17:33:53  spmcmil
 * Modified to use meta functions (meta_write/read/creat/open/close/unlink).
 *
 * Revision 1.5  1998/02/16 19:24:57  rbross
 * Newt patches
 *
 * Revision 1.4  1997/10/28 19:16:15  pvfs
 * added license
 *
 * Revision 1.3  1997/06/10 17:52:18  walt
 * updated directory handling code
 *
 * Revision 1.2  1997/06/09 19:28:40  walt
 * added directory support
 *
 * Revision 1.1  1997/01/16 17:30:31  mcettei
 * Initial revision
 *
 *
/*/

static char md_chown_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/meta/RCS/md_chown.c,v 1.8 1998/08/07 17:28:34 spmcmil Exp $";
/*This file contains the chown metadata file call for the PVFS*/
/*Assumes that space for request has been allocated. */

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/param.h>
#include <linux/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include <grp.h>
#include <pwd.h>

extern int errno;

int md_chown(request)
mreq_p request;
{
fmeta meta1;
int fd,length,i;
char temp[MAXPATHLEN];
dmeta dir;
struct passwd *pw;
struct group *grp;


	if ((fd = meta_open(request->req.chown.fname, O_RDWR)) < 0) {
		if ((int)errno != (int)EISDIR) {
			perror("md_chown: meta_open");
			return (-1);
		}
	}

   /* check execute permissions on directory */
   strncpy(temp, request->req.open.fname, MAXPATHLEN);
	length = get_parent(temp);
	/* length < 0 means CWD */
   if (length >= 0) {
		if (meta_access(0, temp, request->uid, request->gid, X_OK) < 0) {
			perror("md_open: meta_access");
			return(-1);
		}
	}
	
	/* Read metadata file */	
	if (fd >= 0) {
		if (meta_read(fd, &meta1) != sizeof(fmeta)) {
			perror("md_chown: meta_read");
			meta_close(fd);
			return (-1);
		}
		lseek(fd, 0, SEEK_SET);
	}
	else { /* directory */
		get_dmeta(request->req.stat.fname, &dir);
		meta1.u_stat.st_uid = dir.dr_uid;
		meta1.u_stat.st_gid = dir.dr_gid;
	}

	/* Change owner if necessary */
	if (request->req.chown.owner >= 0) {
		/* root can change this or if uid = file owner uid,
		 * let group chown be performed */
		if ((request->uid != 0) && (request->uid != request->req.chown.owner)) {
			errno = EPERM;
			perror("md_chown: permission denied");
			if (fd >= 0)
				meta_close(fd);
			return(-1);
		}
		meta1.u_stat.st_uid = request->req.chown.owner;
	}
	
	/* Change group owner if necessary */
	if (request->req.chown.group >= 0) {
		/* check to see if user is member of group! -- easier way anybody? */
		/* get group entry from /etc/group file. */
		while((grp = getgrent()) && grp->gr_gid != request->req.chown.group);
		endgrent(); /* close /etc/group file */
		/* see if user is a member in /etc/group or /etc/passwd */
		pw = getpwuid(request->uid);
		for(i=0;grp->gr_mem[i] && strcmp(pw->pw_name, grp->gr_mem[i]);
			i++);
		/* if not root, and if not member of group being changed to, fail */
		if ((request->uid != 0) && !(grp->gr_mem[i]) &&
			(request->req.chown.group != pw->pw_gid)) {
			errno = EPERM;
			perror("md_chown: change group permission denied");
			if (fd >= 0)
				meta_close(fd);
			return(-1);
		}
		meta1.u_stat.st_gid = request->req.chown.group;
	}	

	/* Write metadata back */
	if (fd >= 0) {
		if (meta_write(fd, &meta1, sizeof(fmeta)) != sizeof(fmeta)) {
			perror("md_chown: meta_write.");
			meta_close(fd);
			return(-1);
		}
	}
	else { /* directory */
		dir.dr_uid = meta1.u_stat.st_uid;
		dir.dr_gid = meta1.u_stat.st_gid;
		put_dmeta(request->req.stat.fname, &dir);
	}

	/* Close metadata file */
	if (fd >= 0 && (meta_close(fd)) < 0) {
		perror("md_chown: meta_close");
		return(-1);
	}	

	/* Do acknowledge and return */
	return (0);

}	
