/*
/* md_close.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* MD_CLOSE.C - 
/*
/*/

/*
 * $Log: md_close.c,v $
 * Revision 1.7  1998/08/10 19:32:10  spmcmil
 * Took out meta_access.  No need to check for permission to close
 * a file.
 *
 * Revision 1.6  1998/08/10 19:22:16  spmcmil
 * Now works with ctime,atime, and mtime.
 *
 * Revision 1.4  1998/07/20 17:33:53  spmcmil
 * Modified to use meta functions (meta_write/read/creat/open/close/unlink).
 *
 * Revision 1.3  1998/02/16 19:24:57  rbross
 * Newt patches
 *
 * Revision 1.2  1997/10/28 19:16:27  pvfs
 * added license
 *
 * Revision 1.1  1997/01/16 17:32:05  mcettei
 * Initial revision
 *
 *
/*/

static char md_close_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/meta/RCS/md_close.c,v 1.7 1998/08/10 19:32:10 spmcmil Exp $";
/*This file contains the close metadata file call for the PVFS*/
/*File assumes request has been allocated */

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <linux/types.h>
#include <sys/timeb.h>
#include <fcntl.h>
#include <unistd.h>
#include <meta.h>
#include <req.h>
#include <syscall.h>

extern int errno;

int md_close(req, fname)
mreq_p req;
char *fname;
{
	time_t ctime;
	fmeta meta;
	int fd;

	/* get fd for meta file */
	if ((fd = meta_open(fname, O_RDWR)) < 0) {
		perror("md_close: meta_open");
		return(-1);
	}

	/* get metadata */
	if (meta_read(fd, &meta) < 0) {
		perror("md_close: meta_read");
		return(-1);
	}

	/* seek back to position 0 */
	if (syscall(SYS_lseek, fd, 0, SEEK_SET) < 0) return(-1);

	/* get current time for meta stats */
	if ((ctime = time(NULL)) < 0) {
		perror("md_close: time");
		return(-1);
	}

	/* setup times */
	/* mtime and atime are changed by pvfs_write and pvfs_read
	 * respectively, while this function explicitly modifies ctime. */
	meta.u_stat.st_atime = req->req.close.meta.u_stat.st_atime;
	meta.u_stat.st_mtime = req->req.close.meta.u_stat.st_mtime;
	meta.u_stat.st_ctime = ctime; /* is this correct semantics? */

/* Write metadata back */
	if (meta_write(fd, &meta, sizeof(fmeta)) != sizeof(fmeta)) {
		perror("md_close: meta_write.");
		return(-1);
	}	

/* Close metadata file */
	if (meta_close(fd) < 0) {
		perror ("File close");
		return(-1);
	}
	
	return (0);

}	
