/*
 * md_open.c copyright (c) 1996,1997 Clemson University, all rights reserved.
 *
 * Written by Rob Ross and Matt Cettei.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.eng.clemson.edu
 *           Matt Cettei mcettei@parl.eng.clemson.edu
 */

/*
 * $Log: md_open.c,v $
 * Revision 1.10  1998/08/07 17:29:02  spmcmil
 * Modified meta_access stuff and cleaned up.
 *
 * Revision 1.8  1998/07/21 04:22:59  rbross
 * major rewrite; still needs serious work
 *
 * Revision 1.7  1998/07/20 17:33:53  spmcmil
 * Modified to use meta functions (meta_write/read/creat/open/close/unlink).
 *
 * Revision 1.6  1998/07/09 16:54:47  rbross
 * fixed bug in open WRT base > 0
 *
 * Revision 1.5  1998/02/16 19:29:21  rbross
 * Newt patches
 *
 * Revision 1.4  1997/11/10 19:52:33  rbross
 * *** empty log message ***
 *
 * Revision 1.3  1997/03/19 21:12:32  mcettei
 * *** empty log message ***
 *
 * Revision 1.2  1997/02/25 18:58:31  pvfs
 * changed error messages
 *
 * Revision 1.1  1997/01/16 17:36:21  mcettei
 * Initial revision
 *
 *
/*/

static char md_open_c[] = "$Header: /projects/pvfs/pvfs-working/mgr/meta/RCS/md_open.c,v 1.10 1998/08/07 17:29:02 spmcmil Exp $";
/* This file contains the open metadata file call for the PVFS	* 
 * Assumes request and metar are previously allocated. 		  	* 
 * rootbuf is a static char string.  The root directory of 		* 
 * the file system is determined from the .pvfsdir file and 	* 
 * returned to the manager.												* 
 *	If file does not exist, it will be created, but only if  	* 
 * the pvfs dot file is present for the directory.             * 
 */

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <linux/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <meta.h>
#include <req.h>
#include <debug.h>
#include <syscall.h>
#include <sys/param.h>

/* DEFINES */
#define RPSTAT request->req.open.meta.p_stat

/* GLOBALS */
extern int errno;

/* PROTOTYPES */

/* MD_OPEN() - opens (and may create) metadata files for the manager.
 * This code really, really sucks.  I'm ashamed of it.  There is
 * little/no permission checking going on, and there are pointer
 * oddities as well WRT metar_p.  I don't like it.  Some day it should
 * be rewritten...
 */
int md_open(request, metar_p)
mreq_p request;
fmeta_p metar_p;
{
int uid, gid, fd, length, mode;
fmeta meta1;
char temp[MAXPATHLEN];

	/* open meta data file (this will not create it!) */
	if ((fd = meta_open(request->req.open.fname, O_RDONLY)) < 0) {

		/* ok, something happened...were we trying to create a new file? */
		if (!(request->req.open.flag & O_CREAT)) {
			/* file does not exist and we aren't trying to create file! */
			fprintf(stderr, "md_open: meta_open (%s): %s\n",
				request->req.open.fname, strerror(errno));
			return(-1);
		}

		/* strip off file name so we can check write permissions */
		strncpy(temp, request->req.open.fname, MAXPATHLEN);
		for (length=strlen(temp);length >= 0 && temp[length] != '/';length--);
		/* if length<0, CWD is being used and directory permissions will 
		 *	not be checked pursuant to the UNIX method */
		if (length >= 0) {
			temp[length] = '\0'; /* cut off file name */
			/* check for permissions to write to directory */
			if (meta_access(0, temp, request->uid, request->gid, X_OK | W_OK)
				< 0) {
				perror("md_open: meta_access");
				return(-1);
			}
		}

		/* Yep...need to try to create the file...check params first */
		if ((RPSTAT.base < 0) || (RPSTAT.pcount < 0)
		|| (RPSTAT.ssize < 0) || (RPSTAT.soff < 0)
		|| (RPSTAT.bsize < 0) || (RPSTAT.pcount > 65535))
		{
			LOG("Invalid pstat values");
			errno = EINVAL;
			return(-1);
		}

		/* now create meta file */
		if ((fd = meta_creat(request->req.open.fname, O_RDWR)) < 0) {
			fprintf(stderr, "md_open: meta_creat (%s): %s\n",
				request->req.open.fname, strerror(errno));
			return(-1);
		}

		/* fill in the metadata and exit...again, should be permission
		 * checking going on...
		 */
		uid = request->req.open.meta.u_stat.st_uid;
		gid = request->req.open.meta.u_stat.st_gid;
		if (unix_fstat(fd, &(request->req.open.meta.u_stat)) < 0) {
			perror("Error doing fstat");
			return (-1);
		}	
		request->req.open.meta.u_stat.st_uid = uid;
		request->req.open.meta.u_stat.st_gid = gid;
		if (meta_write(fd, &(request->req.open.meta),
			sizeof(fmeta)) != sizeof(fmeta))
		{ 
			perror("Error writing created file.");
			return (-1);
		}	
		*metar_p = request->req.open.meta;
		meta_close(fd);
		return(0);
	}

	/* check for read/write permissions here */
	mode = R_OK | W_OK | X_OK; /* default mode (SHOULD never happen!)*/
	switch (request->req.open.mode && O_ACCMODE) {
		case 0:	mode = R_OK; /* O_RDONLY */
					break;
		case 1:	mode = W_OK; /* O_WRONLY */
					break;
		case 2:	mode = R_OK | W_OK; /* O_RDWR */
					break;
	}
	if (meta_access(fd, request->req.open.fname, request->uid, 
		request->gid, mode) < 0) {
		perror("md_open: meta_access");
		return(-1);
	}

	/* wow -- we have the file open... */
	if (meta_read(fd, &meta1, sizeof(fmeta)) != sizeof(fmeta))  {
 		perror("File read");
		return (-1);
	}
	meta_close(fd);
	*metar_p = meta1;
	return (0);
}

#undef RPSTAT
