/*
 * get_dmeta.c copyright (c) 1997 Clemson University, all rights reserved.
 *
 * Written by Matt Cettei and Walt Ligon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.eng.clemson.edu
 *           Matt Cettei mcettei@parl.eng.clemson.edu
 */

/*
 * $Log: get_dmeta.c,v $
 * Revision 1.7  1998/08/03 20:40:33  rbross
 * rewritten to use dmetaio calls
 *
 * Revision 1.6  1998/07/10 20:21:05  rbross
 * cleanup
 *
 * Revision 1.5  1998/02/21 03:29:16  rbross
 * yet another stat fix
 *
 * Revision 1.4  1998/02/19 23:03:06  rbross
 * removed SYS_stat defines ... moving to lib.h
 *
 * Revision 1.3  1998/02/16 22:03:56  rbross
 * repatched to work with libc5 too
 *
 * Revision 1.2  1997/06/26 22:22:04  walt
 * fixed problem getting dmeta for a directory
 *
 * Revision 1.1  1997/06/10 17:52:18  walt
 * Initial revision
 *
/*/

static char get_dmeta_c[] = "$Header: /projects/pvfs/pvfs-working/shared/RCS/get_dmeta.c,v 1.7 1998/08/03 20:40:33 rbross Exp $";

#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <syscall.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <meta.h>
#include <sys/param.h>

#define TMPSZ 512

/* GLOBALS */
extern int errno;
static char temp[TMPSZ];

/* get_dmeta() - grabs directory metadata in the directory of a given
 * file
 *
 * NOTE: metadata is only valid until next call to get_dmeta()
 */
int get_dmeta(char * fname, dmeta_p dir) {
	int fd, len, is_dir = 0, use_cwd = 0;
	struct stat filestat;
	char nbuf[MAXPATHLEN];
	static char dmbuf[4096];
	struct dmeta *d_p;

	strncpy(nbuf, fname, MAXPATHLEN);

	/* strip off any trailing /'s on the file name */
	for (len = strlen(nbuf); len > 0 && nbuf[len] == '/'; nbuf[len--] = '\0');
	if (len < 0) return(-1);

	/* determine if name refers to a directory or file, find appropriate
	 * directory name
	 */
	if (unix_stat(nbuf, &filestat) < 0 || !S_ISDIR(filestat.st_mode)) {
		/* need to strip off trailing name, pass the rest to dmetaio
		 * calls so we get the dir. metadata for the parent directory
		 *
		 * we don't want to wax the last bit of the name, because we need
		 * it later...
		 */
		for (/* len already set */; len >= 0 && nbuf[len] != '/'; len--);
		if (len <= 0) /* no directory; use current working directory */ {
			use_cwd = 1;
		}
		else /* the fname wasn't just a file name */ {
			nbuf[len] = '\0';
		}
	}
	else /* it's a directory */ {
		is_dir = 1;
	}
	
	/* open the dir. metadata file, read it, get the info */
	if (use_cwd) {
		if ((fd = dmeta_open(".", O_RDONLY)) < 0) {
			perror("dmeta_open (.)");
			return(-1);
		}
	}
	else if ((fd = dmeta_open(nbuf, O_RDONLY)) < 0) {
		perror("dmeta_open");
		return(-1);
	}
	if (dmeta_read(fd, dmbuf, 4096) < 0) {
		perror("dmeta_read");
		return(-1);
	}
	dmeta_close(fd);

	/* fill in the filename part of the dmeta structure */
	/* this is a big hack to keep someone's idiot code from breaking */
	d_p = (struct dmeta *) dmbuf;

	d_p->fname = d_p->sd_path + strlen(d_p->sd_path)+1;
	if (is_dir) {
		*d_p->fname = '\0';
	}
	else {
		strcpy(d_p->fname, &nbuf[len+1]); /* could check len... */
	}

	/* copy the dmeta structure to the specified location */
	*dir = *(struct dmeta *)dmbuf;

	return(0);
}
