/*
/* check_sock.c copyright (c) Clemson University, all rights reserved
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Matt Cettei mcettei@parl.eng.clemson.edu

/*/

/*
/* CHECK_SOCK.C - globals and functions used to translate generic socket
/*					 selection routines into platform specific calls
/*/

/*
 * $Log: sockset.c,v $
 * Revision 1.3  1999/02/19 19:10:12  rbross
 * EINTR friendly
 *
 * Revision 1.2  1997/10/28 19:37:21  pvfs
 * added license
 *
 * Revision 1.1  1996/11/25 22:37:50  rbross
 * Initial revision
 *
/*/

static char check_sock_c[]="$Header: /projects/pvfs/pvfs-working/shared/RCS/sockset.c,v 1.3 1999/02/19 19:10:12 rbross Exp $";

/* UNIX INCLUDES */
#include <stdio.h>
#include <errno.h>

/* PVFS INCLUDES */
#include <sockset.h>

/* GLOBALS */
extern int errno;

/* FUNCTIONS */

void addsockrd(int sock, sockset_p set_p)
{
	FD_SET(sock, &set_p->read);
	/* max_index MUST equal (max. sock # + 1) */
	if (sock >= set_p->max_index) set_p->max_index = sock+1;
}

void addsockwr(int sock, sockset_p set_p)
{
	FD_SET(sock, &set_p->write);
	if (sock >= set_p->max_index) set_p->max_index = sock+1;
}

void delsock(int sock, sockset_p set_p)
{
	/* make sure we're not going to check for this any more */
	FD_CLR(sock, &set_p->read);
	FD_CLR(sock, &set_p->write);
	FD_CLR(sock, &set_p->tmp_read);
	FD_CLR(sock, &set_p->tmp_write);

	if (sock + 1 >= set_p->max_index) /* need to reduce max_index */ {
		int i;
		for (i = sock; i >= 0; i--)
			if (FD_ISSET(i, &set_p->read) || FD_ISSET(i, &set_p->write)) break;
		set_p->max_index = i+1;
	}
}

int nextsock(sockset_p set_p)
{
	int i = 0;
	if (set_p->max_index <= 0) /* empty set */ return(-1);
	while (i < set_p->max_index && !FD_ISSET(i, &set_p->tmp_read) &&
		!FD_ISSET(i, &set_p->tmp_write)) i++;
	if (i == set_p->max_index) return(-1);
	else {
		FD_CLR(i, &set_p->tmp_read);
		FD_CLR(i, &set_p->tmp_write);
		return(i);
	}  
} /* end of nextsock() */

int dumpsocks(sockset_p set_p)
{
	int i;
	fprintf(stderr, "max_index = %d\n", set_p->max_index);
	for (i = 0; i < set_p->max_index; i++) {
		if (FD_ISSET(i, &set_p->read)) fprintf(stderr, "%d:\tREAD\n", i);
		if (FD_ISSET(i, &set_p->write)) fprintf(stderr, "%d:\tWRITE\n", i);
	} 
} /* end of DUMPSOCKS() -- select() version */


void initset(sockset_p set_p)
{
	set_p->max_index = 0;
	FD_ZERO(&set_p->read);
	FD_ZERO(&set_p->write);
	FD_ZERO(&set_p->tmp_read);
	FD_ZERO(&set_p->tmp_write);
}

/* time given in msec */
/* Returns # of sockets in descriptor set (?), or -1 on error
 */
int check_socks(sockset_p set_p, int time)
{
	int ret;

check_socks_restart:
	set_p->tmp_read  = set_p->read;
	set_p->tmp_write = set_p->write;
	if (time == -1) ret = select(set_p->max_index, &set_p->tmp_read,
		&set_p->tmp_write, NULL, NULL);
	else {
		set_p->timeout.tv_sec = time / 1000;
		set_p->timeout.tv_usec = (time % 1000) * 1000;
		ret = select(set_p->max_index, &set_p->tmp_read,
			&set_p->tmp_write, NULL, &set_p->timeout);
	}
	if (ret < 0 && errno == EINTR) goto check_socks_restart;
	return(ret);
}
