/*
 * unix-stats.c copyright (c) 1997 Clemson University, all rights reserved.
 *
 * Written by Rob Ross and Matt Cettei.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Contact:  Rob Ross    rbross@parl.eng.clemson.edu
 *           Matt Cettei mcettei@parl.eng.clemson.edu
 */

/*
 * unix-stats.c - wrapper for syscall()s that perform the various stat
 * calls, using the newer syscall() parameters, which require the use
 * of struct new_stat instead of struct stat.
 *
 * Basically the problem is that there is some wack stuff going on with
 * respect to glibc2 (libc6), the linux kernel, syscall, and the stat
 * calls.  This is all here to fix that and give us a place to handle
 * any other problems related to the stat calls that might turn up.
 *
 *
 * 3/19/99 - update on this stat issue
 * The previous (2.0.xx) kernel had two stat structures defined in
 * /usr/include/asm/stat.h, "old_stat" and "new_stat", which we had to
 * convert between.  Now (2.[12].xx) there is "__old_kernel_stat" and
 * "stat" in there.  In both cases we have to do conversion to a new
 * format from an old one.
 *
 * It turns out that the glibc source addresses this in a similar way.
 * What I'm going to do here is basically include the glibc stuff...this
 * should be portable across the linux kernels but that's it.  Better
 * than what we WERE doing <smile>...
 *
 * Thanks to Maciek (maciek@cacr.caltech.edu) for helping me figure this
 * out.
 *
 */

/*
 * $Log: unix-stats.c,v $
 * Revision 1.2  1999/03/23 16:16:05  rbross
 * updates to work with linux 2.2
 *
 */

#include <syscall.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <linux/types.h>
#include "kernel_stat.h"


int unix_lstat(const char *fn, struct stat *s_p)
{
	struct kernel_stat nstat;

	if (syscall(SYS_lstat, fn, &nstat)) return(-1);

	bzero(s_p, sizeof(struct stat));
	s_p->st_dev     = nstat.st_dev;
	s_p->st_ino     = nstat.st_ino;
	s_p->st_mode    = nstat.st_mode;
	s_p->st_nlink   = nstat.st_nlink;
	s_p->st_uid     = nstat.st_uid;
	s_p->st_gid     = nstat.st_gid;
	s_p->st_rdev    = nstat.st_rdev;
	s_p->st_size    = nstat.st_size;
	s_p->st_blksize = nstat.st_blksize;
	s_p->st_blocks  = nstat.st_blocks;
	s_p->st_atime   = nstat.st_atime;
	s_p->st_mtime   = nstat.st_mtime;
	s_p->st_ctime   = nstat.st_ctime;
	return(0);
}

int unix_stat(const char *fn, struct stat *s_p)
{
	struct kernel_stat nstat;

	if (syscall(SYS_stat, fn, &nstat)) return(-1);

	bzero(s_p, sizeof(struct stat));
	s_p->st_dev     = nstat.st_dev;
	s_p->st_ino     = nstat.st_ino;
	s_p->st_mode    = nstat.st_mode;
	s_p->st_nlink   = nstat.st_nlink;
	s_p->st_uid     = nstat.st_uid;
	s_p->st_gid     = nstat.st_gid;
	s_p->st_rdev    = nstat.st_rdev;
	s_p->st_size    = nstat.st_size;
	s_p->st_blksize = nstat.st_blksize;
	s_p->st_blocks  = nstat.st_blocks;
	s_p->st_atime   = nstat.st_atime;
	s_p->st_mtime   = nstat.st_mtime;
	s_p->st_ctime   = nstat.st_ctime;
	return(0);
}

int unix_fstat(int fd, struct stat *s_p)
{
	struct kernel_stat nstat;

	if (syscall(SYS_fstat, fd, &nstat)) return(-1);

	bzero(s_p, sizeof(struct stat));
	s_p->st_dev     = nstat.st_dev;
	s_p->st_ino     = nstat.st_ino;
	s_p->st_mode    = nstat.st_mode;
	s_p->st_nlink   = nstat.st_nlink;
	s_p->st_uid     = nstat.st_uid;
	s_p->st_gid     = nstat.st_gid;
	s_p->st_rdev    = nstat.st_rdev;
	s_p->st_size    = nstat.st_size;
	s_p->st_blksize = nstat.st_blksize;
	s_p->st_blocks  = nstat.st_blocks;
	s_p->st_atime   = nstat.st_atime;
	s_p->st_mtime   = nstat.st_mtime;
	s_p->st_ctime   = nstat.st_ctime;
	return(0);
}
